## @file
# UnitTest for inf_generator.py
#
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##

import unittest
import tempfile
import os
import datetime
from edk2toollib.windows.capsule.inf_generator import InfGenerator, InfSection


def _get_test_file():
    return TEST_FILE_CONTENTS.replace("<DATESUB>", datetime.date.today().strftime("%m/%d/%Y"))


class InfSectionTest(unittest.TestCase):
    def test_empty(self):
        section = InfSection('TestSection')
        self.assertEqual(str(section), "[TestSection]")

    def test_single(self):
        section = InfSection("Test")
        section.Items.append("Item")
        self.assertEqual(str(section), "[Test]\nItem")

    def test_multiple(self):
        section = InfSection("Test")
        section.Items.append("Item1")
        section.Items.append("Item2")
        self.assertEqual(str(section), "[Test]\nItem1\nItem2")


class InfGeneratorTest(unittest.TestCase):
    VALID_GUID_STRING = "3cad7a0c-d35b-4b75-96b1-03a9fb07b7fc"

    def test_valid(self):
        o = InfGenerator("test_name", "provider", InfGeneratorTest.VALID_GUID_STRING, "x64",
                         "description", "aa.bb.cc.dd", "0xaabbccdd")
        self.assertIsInstance(o, InfGenerator)
        self.assertEqual(o.Name, "test_name")
        self.assertEqual(o.Provider, "provider")
        self.assertEqual(o.EsrtGuid, InfGeneratorTest.VALID_GUID_STRING)
        self.assertEqual(o.Arch, InfGenerator.SUPPORTED_ARCH["x64"])
        self.assertEqual(o.Description, "description")
        self.assertEqual(int(o.VersionHex, 0), int("0xaabbccdd", 0))
        self.assertEqual(o.VersionString, "aa.bb.cc.dd")
        self.assertEqual(o.Manufacturer, "provider")

        # loop thru all supported arch and make sure it works
        for a in InfGenerator.SUPPORTED_ARCH.keys():
            with self.subTest(Arch=a):
                o.Arch = a
                self.assertEqual(InfGenerator.SUPPORTED_ARCH[a], o.Arch)

        # set manufacturer
        o.Manufacturer = "manufacturer"
        self.assertEqual("manufacturer", o.Manufacturer)

    def test_file(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            inffile_path = os.path.join(tmpdir, "InfFile.inf")
            infgen = InfGenerator('TestName', 'TestProvider', InfGeneratorTest.VALID_GUID_STRING,
                                  "x64", "Test Description", "1.2.3.4", "0x01020304")
            infgen.MakeInf(inffile_path, "TestFirmwareRom.bin", False)

            with open(inffile_path, "r") as inffile:
                file_contents = inffile.read()
                # Remove all whitespace, just in case.
                file_contents = file_contents.replace("\n", "").replace("\t", "").replace(" ", "")
                test_contents = _get_test_file().replace("\n", "").replace("\t", "").replace(" ", "")
                self.assertEqual(test_contents, file_contents)

    def test_integrity_file_entry(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            inffile_path = os.path.join(tmpdir, "InfFile.inf")
            infgen = InfGenerator('TestName', 'TestProvider', InfGeneratorTest.VALID_GUID_STRING,
                                  "x64", "Test Description", "1.2.3.4", "0x01020304")
            infgen.MakeInf(inffile_path, "TestFirmwareRom.bin", False)
            with open(inffile_path, "r") as inffile:
                file_contents = inffile.read()
                self.assertNotIn("SampleIntegrityFile.bin", file_contents)
                self.assertNotIn("FirmwareIntegrityFilename", file_contents)

            infgen.IntegrityFilename = "SampleIntegrityFile.bin"
            infgen.MakeInf(inffile_path, "TestFirmwareRom.bin", False)
            with open(inffile_path, "r") as inffile:
                file_contents = inffile.read()
                self.assertIn("SampleIntegrityFile.bin", file_contents)
                self.assertIn("FirmwareIntegrityFilename", file_contents)

    def test_invalid_name_symbol(self):

        InvalidChars = ['~', '`', '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', ' ', '{', '[', '}', ']', '+', '=']
        for a in InvalidChars:
            with self.subTest(name="test{}name".format(a)):
                name = "test{}name".format(a)
                with self.assertRaises(ValueError):
                    InfGenerator(name, "provider", InfGeneratorTest.VALID_GUID_STRING, "x64",
                                 "description", "aa.bb", "0xaabbccdd")

    def test_version_string_format(self):
        with self.subTest(version_string="zero ."):
            with self.assertRaises(ValueError):
                InfGenerator("test_name", "provider", InfGeneratorTest.VALID_GUID_STRING, "x64",
                             "description", "1234", "0x100000000")

        with self.subTest(version_string="> 3 ."):
            with self.assertRaises(ValueError):
                InfGenerator("test_name", "provider", InfGeneratorTest.VALID_GUID_STRING, "x64",
                             "description", "1.2.3.4.5", "0x100000000")

    def test_version_hex_too_big(self):
        with self.subTest("hex string too big"):
            with self.assertRaises(ValueError):
                InfGenerator("test_name", "provider", InfGeneratorTest.VALID_GUID_STRING, "x64",
                             "description", "aa.bb", "0x100000000")

        with self.subTest("decimal too big"):
            with self.assertRaises(ValueError):
                InfGenerator("test_name", "provider", InfGeneratorTest.VALID_GUID_STRING, "x64",
                             "description", "aa.bb", "4294967296")

    def test_version_hex_can_support_decimal(self):
        o = InfGenerator("test_name", "provider", InfGeneratorTest.VALID_GUID_STRING, "x64",
                         "description", "aa.bb.cc.dd", "12356")
        self.assertEqual(int(o.VersionHex, 0), 12356)

    def test_invalid_guid_format(self):
        with self.assertRaises(ValueError):
            InfGenerator("test_name", "provider", "NOT A VALID GUID", "x64", "description", "aa.bb", "0x1000000")


# NOTE: Below are the expected contents of a a valid INF file.
#       Some fields will need to be generated (like the date).
TEST_FILE_CONTENTS = ''';
; TestName.inf
; 1.2.3.4
; Copyright (C) 2019 Microsoft Corporation.  All Rights Reserved.
;
[Version]
Signature="$WINDOWS NT$"
Class=Firmware
ClassGuid={f2e7dd72-6468-4e36-b6f1-6488f42c1b52}
Provider=%Provider%
DriverVer=<DATESUB>,1.2.3.4
PnpLockdown=1
CatalogFile=TestName.cat

[Manufacturer]
%MfgName% = Firmware,NTamd64

[Firmware.NTamd64]
%FirmwareDesc% = Firmware_Install,UEFI\\RES_{3cad7a0c-d35b-4b75-96b1-03a9fb07b7fc}

[Firmware_Install.NT]
CopyFiles = Firmware_CopyFiles

[Firmware_CopyFiles]
TestFirmwareRom.bin

[Firmware_Install.NT.Hw]
AddReg = Firmware_AddReg

[Firmware_AddReg]
HKR,,FirmwareId,,{3cad7a0c-d35b-4b75-96b1-03a9fb07b7fc}
HKR,,FirmwareVersion,%REG_DWORD%,0x1020304
HKR,,FirmwareFilename,,TestFirmwareRom.bin

[SourceDisksNames]
1 = %DiskName%

[SourceDisksFiles]
TestFirmwareRom.bin = 1

[DestinationDirs]
DefaultDestDir = %DIRID_WINDOWS%,Firmware ; %SystemRoot%\\Firmware

[Strings]
; localizable
Provider     = "TestProvider"
MfgName      = "TestProvider"
FirmwareDesc = "Test Description"
DiskName     = "Firmware Update"

; non-localizable
DIRID_WINDOWS = 10
REG_DWORD     = 0x00010001
'''



## @file
# UnitTest for inf_generator.py
#
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##

import unittest
import textwrap
from edk2toollib.windows.capsule.inf_generator2 import InfHeader, InfStrings, InfSourceFiles, InfFirmware
from edk2toollib.windows.capsule.inf_generator2 import InfFirmwareSections, InfFile


class InfHeaderTest(unittest.TestCase):
    def test_header(self):
        Strings = InfStrings()
        Header = InfHeader("InfTest", "1.0.0.1", "01/01/2021", "amd64", "testprovider", "testmfr", Strings)

        ExpectedStr = textwrap.dedent("""\
            ;
            ; InfTest
            ; 1.0.0.1
            ; Copyright (C) Microsoft Corporation. All Rights Reserved.
            ;
            [Version]
            Signature="$WINDOWS NT$"
            Class=Firmware
            ClassGuid={f2e7dd72-6468-4e36-b6f1-6488f42c1b52}
            Provider=%Provider%
            DriverVer=01/01/2021,1.0.0.1
            PnpLockdown=1
            CatalogFile=InfTest.cat

            [Manufacturer]
            %MfgName% = Firmware,NTamd64

            """)
        self.assertEqual(ExpectedStr, str(Header))
        self.assertIn("Provider", Strings.LocalizableStrings)
        self.assertEqual("testprovider", Strings.LocalizableStrings['Provider'])
        self.assertIn("MfgName", Strings.LocalizableStrings)
        self.assertEqual("testmfr", Strings.LocalizableStrings['MfgName'])

    def test_header_should_throw_for_bad_input(self):
        Strings = InfStrings()

        with self.assertRaises(ValueError):
            InfHeader("InfTest ?? bad", "1.0.0.1", "01/01/2021", "amd64", "testprovider", "testmfr", Strings)

        with self.assertRaises(ValueError):
            InfHeader("InfTest", "this is not good", "01/01/2021", "amd64", "testprovider", "testmfr", Strings)

        with self.assertRaises(ValueError):
            InfHeader("InfTest", "1.0.0.1", "foobar", "amd64", "testprovider", "testmfr", Strings)

        with self.assertRaises(ValueError):
            InfHeader("InfTest", "1.0.0.1", "01/01/2021", "foobar", "testprovider", "testmfr", Strings)

        with self.assertRaises(TypeError):
            InfHeader(1, "1.0.0.1", "01/01/2021", "amd64", "testprovider", "testmfr", Strings)

        with self.assertRaises(AttributeError):
            InfHeader("InfTest", "1.0.0.1", "01/01/2021", "amd64", "testprovider", "testmfr", None)


class InfFirmwareTest(unittest.TestCase):
    def test_firmware(self):
        Strings = InfStrings()
        SourceFiles = InfSourceFiles("diskname", Strings)

        Firmware = InfFirmware(
            "tag",
            "desc",
            "34e094e9-4079-44cd-9450-3f2cb7824c97",
            "0x01000001",
            "test.bin",
            Strings,
            SourceFiles)

        ExpectedStr = textwrap.dedent("""\
            [tag_Install.NT]
            CopyFiles = tag_CopyFiles

            [tag_CopyFiles]
            test.bin

            [tag_Install.NT.Hw]
            AddReg = tag_AddReg

            [tag_AddReg]
            HKR,,FirmwareId,,{34e094e9-4079-44cd-9450-3f2cb7824c97}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000001
            HKR,,FirmwareFilename,,test.bin

            """)

        self.assertEqual(ExpectedStr, str(Firmware))
        self.assertEqual(Firmware.Description, "desc")
        self.assertIn("REG_DWORD", Strings.NonLocalizableStrings)
        self.assertEqual("0x00010001", Strings.NonLocalizableStrings['REG_DWORD'])
        self.assertIn("test.bin", SourceFiles.Files)

    def test_rollback_firmware(self):
        Strings = InfStrings()
        SourceFiles = InfSourceFiles("diskname", Strings)

        Firmware = InfFirmware(
            "tag",
            "desc",
            "34e094e9-4079-44cd-9450-3f2cb7824c97",
            "0x01000001",
            "test.bin",
            Strings,
            SourceFiles,
            Rollback=True)

        ExpectedStr = textwrap.dedent("""\
            [tag_Install.NT]
            CopyFiles = tag_CopyFiles
            AddReg = tag_DowngradePolicy_AddReg

            [tag_DowngradePolicy_AddReg]
            HKLM,SYSTEM\\CurrentControlSet\\Control\\FirmwareResources\\{34e094e9-4079-44cd-9450-3f2cb7824c97},Policy,%REG_DWORD%,1

            [tag_CopyFiles]
            test.bin

            [tag_Install.NT.Hw]
            AddReg = tag_AddReg

            [tag_AddReg]
            HKR,,FirmwareId,,{34e094e9-4079-44cd-9450-3f2cb7824c97}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000001
            HKR,,FirmwareFilename,,test.bin

            """)

        self.assertEqual(ExpectedStr, str(Firmware))
        self.assertEqual(Firmware.Description, "desc")
        self.assertIn("REG_DWORD", Strings.NonLocalizableStrings)
        self.assertEqual("0x00010001", Strings.NonLocalizableStrings['REG_DWORD'])
        self.assertIn("test.bin", SourceFiles.Files)

    def test_rollback_firmware_integrity(self):
        Strings = InfStrings()
        SourceFiles = InfSourceFiles("diskname", Strings)

        Firmware = InfFirmware(
            "tag",
            "desc",
            "34e094e9-4079-44cd-9450-3f2cb7824c97",
            "0x01000001",
            "test.bin",
            Strings,
            SourceFiles,
            Rollback=True,
            IntegrityFile="test2.bin")

        ExpectedStr = textwrap.dedent("""\
            [tag_Install.NT]
            CopyFiles = tag_CopyFiles
            AddReg = tag_DowngradePolicy_AddReg

            [tag_DowngradePolicy_AddReg]
            HKLM,SYSTEM\\CurrentControlSet\\Control\\FirmwareResources\\{34e094e9-4079-44cd-9450-3f2cb7824c97},Policy,%REG_DWORD%,1

            [tag_CopyFiles]
            test.bin
            test2.bin

            [tag_Install.NT.Hw]
            AddReg = tag_AddReg

            [tag_AddReg]
            HKR,,FirmwareId,,{34e094e9-4079-44cd-9450-3f2cb7824c97}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000001
            HKR,,FirmwareFilename,,test.bin
            HKR,,FirmwareIntegrityFilename,,test2.bin

            """)

        self.assertEqual(ExpectedStr, str(Firmware))
        self.assertEqual(Firmware.Description, "desc")
        self.assertIn("REG_DWORD", Strings.NonLocalizableStrings)
        self.assertEqual("0x00010001", Strings.NonLocalizableStrings['REG_DWORD'])
        self.assertIn("test.bin", SourceFiles.Files)
        self.assertIn("test2.bin", SourceFiles.Files)

    def test_firmware_should_throw_for_bad_input(self):
        Strings = InfStrings()
        SourceFiles = InfSourceFiles("diskname", Strings)

        with self.assertRaises(ValueError):
            InfFirmware(
                "This is not a valid tag name",
                "desc",
                "34e094e9-4079-44cd-9450-3f2cb7824c97",
                "0x01000001",
                "test.bin",
                Strings,
                SourceFiles,
                Rollback=True,
                IntegrityFile="test2.bin")

        with self.assertRaises(ValueError):
            InfFirmware(
                "Tag",
                "desc",
                "This is not a valid UUID.",
                "0x01000001",
                "test.bin",
                Strings,
                SourceFiles,
                Rollback=True,
                IntegrityFile="test2.bin")

        with self.assertRaises(ValueError):
            InfFirmware(
                "Tag",
                "desc",
                "4e094e9-4079-44cd-9450-3f2cb7824c97",  # a more subtle not-valid UUID.
                "0x01000001",
                "test.bin",
                Strings,
                SourceFiles,
                Rollback=True,
                IntegrityFile="test2.bin")

        with self.assertRaises(ValueError):
            InfFirmware(
                "tag",
                "desc",
                "34e094e9-4079-44cd-9450-3f2cb7824c97",
                "foobar",
                "test.bin",
                Strings,
                SourceFiles,
                Rollback=True,
                IntegrityFile="test2.bin")


class InfFirmwareSectionsTest(unittest.TestCase):
    def test_one_section(self):
        Strings = InfStrings()
        SourceFiles = InfSourceFiles("diskname", Strings)

        Firmware = InfFirmware(
            "tag",
            "desc",
            "34e094e9-4079-44cd-9450-3f2cb7824c97",
            "0x01000001",
            "test.bin",
            Strings,
            SourceFiles)

        Sections = InfFirmwareSections('amd64', Strings)
        Sections.AddSection(Firmware)

        ExpectedStr = textwrap.dedent("""\
            [Firmware.NTamd64]
            %tagDesc% = tag_Install,UEFI\\RES_{34e094e9-4079-44cd-9450-3f2cb7824c97}

            [tag_Install.NT]
            CopyFiles = tag_CopyFiles

            [tag_CopyFiles]
            test.bin

            [tag_Install.NT.Hw]
            AddReg = tag_AddReg

            [tag_AddReg]
            HKR,,FirmwareId,,{34e094e9-4079-44cd-9450-3f2cb7824c97}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000001
            HKR,,FirmwareFilename,,test.bin

            """)

        self.assertEqual(ExpectedStr, str(Sections))
        self.assertIn("tagDesc", Strings.LocalizableStrings)
        self.assertEqual("desc", Strings.LocalizableStrings['tagDesc'])
        self.assertIn("test.bin", SourceFiles.Files)

        self.assertIn("REG_DWORD", Strings.NonLocalizableStrings)
        self.assertEqual("0x00010001", Strings.NonLocalizableStrings['REG_DWORD'])

    def test_two_sections(self):
        Strings = InfStrings()
        SourceFiles = InfSourceFiles("diskname", Strings)

        Firmware1 = InfFirmware(
            "tag1",
            "desc1",
            "34e094e9-4079-44cd-9450-3f2cb7824c97",
            "0x01000001",
            "test1.bin",
            Strings,
            SourceFiles)

        Firmware2 = InfFirmware(
            "tag2",
            "desc2",
            "bec9124f-9934-4ec0-a6ed-b8bc1c91d276",
            "0x01000002",
            "test2.bin",
            Strings,
            SourceFiles)

        Sections = InfFirmwareSections('amd64', Strings)
        Sections.AddSection(Firmware1)
        Sections.AddSection(Firmware2)

        ExpectedStr = textwrap.dedent("""\
            [Firmware.NTamd64]
            %tag1Desc% = tag1_Install,UEFI\\RES_{34e094e9-4079-44cd-9450-3f2cb7824c97}
            %tag2Desc% = tag2_Install,UEFI\\RES_{bec9124f-9934-4ec0-a6ed-b8bc1c91d276}

            [tag1_Install.NT]
            CopyFiles = tag1_CopyFiles

            [tag1_CopyFiles]
            test1.bin

            [tag1_Install.NT.Hw]
            AddReg = tag1_AddReg

            [tag1_AddReg]
            HKR,,FirmwareId,,{34e094e9-4079-44cd-9450-3f2cb7824c97}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000001
            HKR,,FirmwareFilename,,test1.bin

            [tag2_Install.NT]
            CopyFiles = tag2_CopyFiles

            [tag2_CopyFiles]
            test2.bin

            [tag2_Install.NT.Hw]
            AddReg = tag2_AddReg

            [tag2_AddReg]
            HKR,,FirmwareId,,{bec9124f-9934-4ec0-a6ed-b8bc1c91d276}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000002
            HKR,,FirmwareFilename,,test2.bin

            """)

        self.assertEqual(ExpectedStr, str(Sections))
        self.assertIn("tag1Desc", Strings.LocalizableStrings)
        self.assertEqual("desc1", Strings.LocalizableStrings['tag1Desc'])
        self.assertIn("test1.bin", SourceFiles.Files)
        self.assertIn("tag2Desc", Strings.LocalizableStrings)
        self.assertEqual("desc2", Strings.LocalizableStrings['tag2Desc'])
        self.assertIn("test2.bin", SourceFiles.Files)

        self.assertIn("REG_DWORD", Strings.NonLocalizableStrings)
        self.assertEqual("0x00010001", Strings.NonLocalizableStrings['REG_DWORD'])

    def test_firmware_sections_should_throw_for_bad_input(self):
        Strings = InfStrings()

        with self.assertRaises(ValueError):
            InfFirmwareSections('foobar', Strings)


class InfSourceFilesTest(unittest.TestCase):
    def test_source_files(self):
        Strings = InfStrings()
        SourceFiles = InfSourceFiles("diskname", Strings)
        SourceFiles.AddFile("test.bin")
        SourceFiles.AddFile("test2.bin")
        SourceFiles.AddFile("test3.bin")

        ExpectedStr = textwrap.dedent("""\
            [SourceDisksNames]
            1 = %DiskName%

            [SourceDisksFiles]
            test.bin = 1
            test2.bin = 1
            test3.bin = 1

            [DestinationDirs]
            DefaultDestDir = %DIRID_WINDOWS%,Firmware ; %SystemRoot%\\Firmware

            """)

        self.assertEqual(ExpectedStr, str(SourceFiles))
        self.assertIn("DiskName", Strings.LocalizableStrings)
        self.assertEqual("diskname", Strings.LocalizableStrings['DiskName'])
        self.assertIn("DIRID_WINDOWS", Strings.NonLocalizableStrings)
        self.assertEqual("10", Strings.NonLocalizableStrings['DIRID_WINDOWS'])

    def test_source_files_should_throw_on_bad_input(self):
        Strings = InfStrings()
        SourceFiles = InfSourceFiles("diskname", Strings)

        with self.assertRaises(ValueError):
            SourceFiles.AddFile("Who Names Files Like This?.bin")


class InfStringsTest(unittest.TestCase):
    def test_inf_strings(self):
        Strings = InfStrings()
        Strings.AddLocalizableString("DiskName", "Firmware Update")
        Strings.AddLocalizableString("Provider", "Test Provider")
        Strings.AddLocalizableString("Tag1Desc", "Test Firmware")

        Strings.AddNonLocalizableString("DIRID_WINDOWS", "10")
        Strings.AddNonLocalizableString("REG_DWORD", "0x00010001")

        ExpectedStr = textwrap.dedent("""\
            [Strings]
            ; localizable
            DiskName = "Firmware Update"
            Provider = "Test Provider"
            Tag1Desc = "Test Firmware"

            ; non-localizable
            DIRID_WINDOWS = 10
            REG_DWORD     = 0x00010001
            """)

        self.assertEqual(ExpectedStr, str(Strings))

    def test_inf_strings_should_throw_on_bad_input(self):
        Strings = InfStrings()

        with self.assertRaises(TypeError):
            Strings.AddLocalizableString(1, 2)

        with self.assertRaises(ValueError):
            Strings.AddLocalizableString("foo bar", "value")

        with self.assertRaises(ValueError):
            Strings.AddLocalizableString("ThisIsNotAllowed;", "value")

        with self.assertRaises(TypeError):
            Strings.AddNonLocalizableString(1, 2)

        with self.assertRaises(ValueError):
            Strings.AddNonLocalizableString("foo bar", "value")

        with self.assertRaises(ValueError):
            Strings.AddNonLocalizableString("ThisIsNotAllowed;", "value")


class InfFileTest(unittest.TestCase):
    def test_inf_file(self):
        File = InfFile("CapsuleName", "1.0.0.1", "01/01/2021", "Test Provider", "Test Manufacturer")

        File.AddFirmware(
            "tag1",
            "desc1",
            "34e094e9-4079-44cd-9450-3f2cb7824c97",
            "0x01000001",
            "test1.bin")

        File.AddFirmware(
            "tag2",
            "desc2",
            "bec9124f-9934-4ec0-a6ed-b8bc1c91d276",
            "0x01000002",
            "test2.bin")

        ExpectedStr = textwrap.dedent("""\
            ;
            ; CapsuleName
            ; 1.0.0.1
            ; Copyright (C) Microsoft Corporation. All Rights Reserved.
            ;
            [Version]
            Signature="$WINDOWS NT$"
            Class=Firmware
            ClassGuid={f2e7dd72-6468-4e36-b6f1-6488f42c1b52}
            Provider=%Provider%
            DriverVer=01/01/2021,1.0.0.1
            PnpLockdown=1
            CatalogFile=CapsuleName.cat

            [Manufacturer]
            %MfgName% = Firmware,NTamd64

            [Firmware.NTamd64]
            %tag1Desc% = tag1_Install,UEFI\\RES_{34e094e9-4079-44cd-9450-3f2cb7824c97}
            %tag2Desc% = tag2_Install,UEFI\\RES_{bec9124f-9934-4ec0-a6ed-b8bc1c91d276}

            [tag1_Install.NT]
            CopyFiles = tag1_CopyFiles

            [tag1_CopyFiles]
            test1.bin

            [tag1_Install.NT.Hw]
            AddReg = tag1_AddReg

            [tag1_AddReg]
            HKR,,FirmwareId,,{34e094e9-4079-44cd-9450-3f2cb7824c97}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000001
            HKR,,FirmwareFilename,,test1.bin

            [tag2_Install.NT]
            CopyFiles = tag2_CopyFiles

            [tag2_CopyFiles]
            test2.bin

            [tag2_Install.NT.Hw]
            AddReg = tag2_AddReg

            [tag2_AddReg]
            HKR,,FirmwareId,,{bec9124f-9934-4ec0-a6ed-b8bc1c91d276}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000002
            HKR,,FirmwareFilename,,test2.bin

            [SourceDisksNames]
            1 = %DiskName%

            [SourceDisksFiles]
            test1.bin = 1
            test2.bin = 1

            [DestinationDirs]
            DefaultDestDir = %DIRID_WINDOWS%,Firmware ; %SystemRoot%\\Firmware

            [Strings]
            ; localizable
            DiskName = "Firmware Update"
            Provider = "Test Provider"
            MfgName  = "Test Manufacturer"
            tag1Desc = "desc1"
            tag2Desc = "desc2"

            ; non-localizable
            DIRID_WINDOWS = 10
            REG_DWORD     = 0x00010001
            """)

        self.assertEqual(ExpectedStr, str(File))

    def test_inf_file_rollback(self):
        File = InfFile("CapsuleName", "1.0.0.1", "01/01/2021", "Test Provider", "Test Manufacturer")

        File.AddFirmware(
            "tag1",
            "desc1",
            "34e094e9-4079-44cd-9450-3f2cb7824c97",
            "0x01000001",
            "test1.bin",
            Rollback=True)

        File.AddFirmware(
            "tag2",
            "desc2",
            "bec9124f-9934-4ec0-a6ed-b8bc1c91d276",
            "0x01000002",
            "test2.bin",
            Rollback=True)

        ExpectedStr = textwrap.dedent("""\
            ;
            ; CapsuleName
            ; 1.0.0.1
            ; Copyright (C) Microsoft Corporation. All Rights Reserved.
            ;
            [Version]
            Signature="$WINDOWS NT$"
            Class=Firmware
            ClassGuid={f2e7dd72-6468-4e36-b6f1-6488f42c1b52}
            Provider=%Provider%
            DriverVer=01/01/2021,1.0.0.1
            PnpLockdown=1
            CatalogFile=CapsuleName.cat

            [Manufacturer]
            %MfgName% = Firmware,NTamd64

            [Firmware.NTamd64]
            %tag1Desc% = tag1_Install,UEFI\\RES_{34e094e9-4079-44cd-9450-3f2cb7824c97}
            %tag2Desc% = tag2_Install,UEFI\\RES_{bec9124f-9934-4ec0-a6ed-b8bc1c91d276}

            [tag1_Install.NT]
            CopyFiles = tag1_CopyFiles
            AddReg = tag1_DowngradePolicy_AddReg

            [tag1_DowngradePolicy_AddReg]
            HKLM,SYSTEM\\CurrentControlSet\\Control\\FirmwareResources\\{34e094e9-4079-44cd-9450-3f2cb7824c97},Policy,%REG_DWORD%,1

            [tag1_CopyFiles]
            test1.bin

            [tag1_Install.NT.Hw]
            AddReg = tag1_AddReg

            [tag1_AddReg]
            HKR,,FirmwareId,,{34e094e9-4079-44cd-9450-3f2cb7824c97}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000001
            HKR,,FirmwareFilename,,test1.bin

            [tag2_Install.NT]
            CopyFiles = tag2_CopyFiles
            AddReg = tag2_DowngradePolicy_AddReg

            [tag2_DowngradePolicy_AddReg]
            HKLM,SYSTEM\\CurrentControlSet\\Control\\FirmwareResources\\{bec9124f-9934-4ec0-a6ed-b8bc1c91d276},Policy,%REG_DWORD%,1

            [tag2_CopyFiles]
            test2.bin

            [tag2_Install.NT.Hw]
            AddReg = tag2_AddReg

            [tag2_AddReg]
            HKR,,FirmwareId,,{bec9124f-9934-4ec0-a6ed-b8bc1c91d276}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000002
            HKR,,FirmwareFilename,,test2.bin

            [SourceDisksNames]
            1 = %DiskName%

            [SourceDisksFiles]
            test1.bin = 1
            test2.bin = 1

            [DestinationDirs]
            DefaultDestDir = %DIRID_WINDOWS%,Firmware ; %SystemRoot%\\Firmware

            [Strings]
            ; localizable
            DiskName = "Firmware Update"
            Provider = "Test Provider"
            MfgName  = "Test Manufacturer"
            tag1Desc = "desc1"
            tag2Desc = "desc2"

            ; non-localizable
            DIRID_WINDOWS = 10
            REG_DWORD     = 0x00010001
            """)

        self.assertEqual(ExpectedStr, str(File))

    def test_inf_file_rollback_integrity(self):
        File = InfFile("CapsuleName", "1.0.0.1", "01/01/2021", "Test Provider", "Test Manufacturer")

        File.AddFirmware(
            "tag1",
            "desc1",
            "34e094e9-4079-44cd-9450-3f2cb7824c97",
            "0x01000001",
            "test1.bin",
            Rollback=True,
            IntegrityFile="integrity1.bin")

        File.AddFirmware(
            "tag2",
            "desc2",
            "bec9124f-9934-4ec0-a6ed-b8bc1c91d276",
            "0x01000002",
            "test2.bin",
            Rollback=True,
            IntegrityFile="integrity2.bin")

        ExpectedStr = textwrap.dedent("""\
            ;
            ; CapsuleName
            ; 1.0.0.1
            ; Copyright (C) Microsoft Corporation. All Rights Reserved.
            ;
            [Version]
            Signature="$WINDOWS NT$"
            Class=Firmware
            ClassGuid={f2e7dd72-6468-4e36-b6f1-6488f42c1b52}
            Provider=%Provider%
            DriverVer=01/01/2021,1.0.0.1
            PnpLockdown=1
            CatalogFile=CapsuleName.cat

            [Manufacturer]
            %MfgName% = Firmware,NTamd64

            [Firmware.NTamd64]
            %tag1Desc% = tag1_Install,UEFI\\RES_{34e094e9-4079-44cd-9450-3f2cb7824c97}
            %tag2Desc% = tag2_Install,UEFI\\RES_{bec9124f-9934-4ec0-a6ed-b8bc1c91d276}

            [tag1_Install.NT]
            CopyFiles = tag1_CopyFiles
            AddReg = tag1_DowngradePolicy_AddReg

            [tag1_DowngradePolicy_AddReg]
            HKLM,SYSTEM\\CurrentControlSet\\Control\\FirmwareResources\\{34e094e9-4079-44cd-9450-3f2cb7824c97},Policy,%REG_DWORD%,1

            [tag1_CopyFiles]
            test1.bin
            integrity1.bin

            [tag1_Install.NT.Hw]
            AddReg = tag1_AddReg

            [tag1_AddReg]
            HKR,,FirmwareId,,{34e094e9-4079-44cd-9450-3f2cb7824c97}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000001
            HKR,,FirmwareFilename,,test1.bin
            HKR,,FirmwareIntegrityFilename,,integrity1.bin

            [tag2_Install.NT]
            CopyFiles = tag2_CopyFiles
            AddReg = tag2_DowngradePolicy_AddReg

            [tag2_DowngradePolicy_AddReg]
            HKLM,SYSTEM\\CurrentControlSet\\Control\\FirmwareResources\\{bec9124f-9934-4ec0-a6ed-b8bc1c91d276},Policy,%REG_DWORD%,1

            [tag2_CopyFiles]
            test2.bin
            integrity2.bin

            [tag2_Install.NT.Hw]
            AddReg = tag2_AddReg

            [tag2_AddReg]
            HKR,,FirmwareId,,{bec9124f-9934-4ec0-a6ed-b8bc1c91d276}
            HKR,,FirmwareVersion,%REG_DWORD%,0x1000002
            HKR,,FirmwareFilename,,test2.bin
            HKR,,FirmwareIntegrityFilename,,integrity2.bin

            [SourceDisksNames]
            1 = %DiskName%

            [SourceDisksFiles]
            test1.bin = 1
            integrity1.bin = 1
            test2.bin = 1
            integrity2.bin = 1

            [DestinationDirs]
            DefaultDestDir = %DIRID_WINDOWS%,Firmware ; %SystemRoot%\\Firmware

            [Strings]
            ; localizable
            DiskName = "Firmware Update"
            Provider = "Test Provider"
            MfgName  = "Test Manufacturer"
            tag1Desc = "desc1"
            tag2Desc = "desc2"

            ; non-localizable
            DIRID_WINDOWS = 10
            REG_DWORD     = 0x00010001
            """)

        self.assertEqual(ExpectedStr, str(File))



# @file junit_report_format_test.py
# Contains unit test routines for the junit_report_format module.
#
#
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##

import unittest
import tempfile
import shutil
import os
import xml.dom.minidom
from edk2toollib.log import junit_report_format


class TestJunitTestReport(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        self.test_dir = None
        super().__init__(*args, **kwargs)

    def prep_workspace(self):
        self.clean_workspace()
        self.test_dir = tempfile.mkdtemp()

    def clean_workspace(self):
        if self.test_dir is None:
            return
        if os.path.isdir(self.test_dir):
            shutil.rmtree(self.test_dir)
            self.test_dir = None

    def setUp(self):
        self.prep_workspace()

    def IsXmlFileValidXml(self, filepath):
        try:
            # this will fail in xml is invalid
            xml.dom.minidom.parse(filepath)
        except Exception as e:
            return (False, str(e))
        return (True, "")

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def tearDown(self):
        self.clean_workspace()

    def test_valid(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.LogStdError("Message to standard error on failed case")
        jtc.SetFailed("Testing the failure path", "TEST")
        jtc2 = jts.create_new_testcase("test_success", "testcase.test.success")
        jtc2.LogStdOut("Message to standard out on success case")
        jtc2.SetSuccess()
        jtc3 = jts.create_new_testcase("test_skipped", "testcase.test.skipped")
        jtc3.SetSkipped()
        jtc4 = jts.create_new_testcase("test_error", "testcase.test.error")
        jtc4.SetError("Testing the error path", "exception")
        f = os.path.join(self.test_dir, "testoutput.xml")
        jr.Output(f)
        self.assertTrue(os.path.isfile(f))

    def test_incomplete_test_results(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.LogStdError("Message to standard error on failed case")
        # Don't set State jtc.SetFailed("Testing the failure path", "TEST")
        f = os.path.join(self.test_dir, "testoutput.xml")
        with self.assertRaises(Exception):
            jr.Output(f)

    def test_xml_escape_in_testcase_name_classname_output(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed\"<", ">&'testcase.test.failed")
        jtc.SetSuccess()
        f = os.path.join(self.test_dir, "testoutput.xml")
        jr.Output(f)
        self.assertTrue(os.path.isfile(f))
        Valid, Msg = self.IsXmlFileValidXml(f)
        self.assertTrue(Valid, msg=Msg)

    def test_xml_escape_in_testsuit_name_package_output(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname\"<", ">&'testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.SetSuccess()
        f = os.path.join(self.test_dir, "testoutput.xml")
        jr.Output(f)
        self.assertTrue(os.path.isfile(f))
        Valid, Msg = self.IsXmlFileValidXml(f)
        self.assertTrue(Valid, msg=Msg)

    def test_xml_escape_in_std_error_output(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.LogStdError("Message to standard error on failed case with < char")
        jtc.SetSuccess()
        f = os.path.join(self.test_dir, "testoutput.xml")
        jr.Output(f)
        self.assertTrue(os.path.isfile(f))
        Valid, Msg = self.IsXmlFileValidXml(f)
        self.assertTrue(Valid, msg=Msg)

    def test_xml_escape_in_std_out_output(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.LogStdOut("Message to \"standard out> ' on failed case with < & char")
        jtc.SetSuccess()
        f = os.path.join(self.test_dir, "testoutput.xml")
        jr.Output(f)
        self.assertTrue(os.path.isfile(f))
        Valid, Msg = self.IsXmlFileValidXml(f)
        self.assertTrue(Valid, msg=Msg)

    def test_xml_escape_in_failed_msg(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.SetFailed("Message in failed case' with < & char", "Error")
        f = os.path.join(self.test_dir, "testoutput.xml")
        jr.Output(f)
        self.assertTrue(os.path.isfile(f))
        Valid, Msg = self.IsXmlFileValidXml(f)
        self.assertTrue(Valid, msg=Msg)

    def test_xml_escape_in_failed_type(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.SetFailed("Message in failed case", "'Fa>ile\"d<")
        f = os.path.join(self.test_dir, "testoutput.xml")
        jr.Output(f)
        self.assertTrue(os.path.isfile(f))
        Valid, Msg = self.IsXmlFileValidXml(f)
        self.assertTrue(Valid, msg=Msg)

    def test_xml_escape_in_error_msg(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_error", "testcase.test.error")
        jtc.SetError("Message in Error\" case with < & char", "Error")
        f = os.path.join(self.test_dir, "testoutput.xml")
        jr.Output(f)
        self.assertTrue(os.path.isfile(f))
        Valid, Msg = self.IsXmlFileValidXml(f)
        self.assertTrue(Valid, msg=Msg)

    def test_xml_escape_in_error_type(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_error", "testcase.test.error")
        jtc.SetError("Message in Error case", 'Error&')
        f = os.path.join(self.test_dir, "testoutput.xml")
        jr.Output(f)
        self.assertTrue(os.path.isfile(f))
        Valid, Msg = self.IsXmlFileValidXml(f)
        self.assertTrue(Valid, msg=Msg)

    def test_set_success_after_set(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.SetFailed("Testing the failure path", "TEST")
        with self.assertRaises(Exception):
            jtc.SetSuccess()

    def test_set_skipped_after_set(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.SetFailed("Testing the failure path", "TEST")
        with self.assertRaises(Exception):
            jtc.SetSkipped()

    def test_set_error_after_set(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_failed", "testcase.test.failed")
        jtc.SetFailed("Testing the failure path", "TEST")
        with self.assertRaises(Exception):
            jtc.SetError("Bad test", "Bad Type")

    def test_set_failure_after_set(self):
        jr = junit_report_format.JunitTestReport()
        jts = jr.create_new_testsuite("testname", "testpackage")
        jtc = jts.create_new_testcase("test_success", "testcase.test.success")
        jtc.SetSuccess()
        with self.assertRaises(Exception):
            jtc.SetFailed("Testing the failure path", "TEST")



##
# Copyright (C) Microsoft Corporation. All rights reserved.
# SPDX-License-Identifier: BSD-2-Clause-Patent
#
# Python script that converts a raw IVRS table into a struct
##
# spell-checker:ignore IVMD, IOMMUEFR

import unittest
from edk2toollib.acpi.ivrs_parser import IVRS_TABLE


class IvrsParserTest(unittest.TestCase):

    dte_00h = None
    dte_01h = None
    dte_02h = None
    dte_03h = None
    dte_42h = None
    dte_43h = None
    dte_46h = None
    dte_47h = None
    dte_48h = None
    dte_f0h_0 = None
    dte_f0h_1 = None
    dte_f0h_2 = None

    ivhd_10h = None
    ivhd_11h = None
    ivhd_40h = None

    ivmd_20h = None
    ivmd_21h = None
    ivmd_22h = None

    ivrs_table = None

    # All the dte types start with Type (1 byte), DeviceID (2 bytes), DTE Settings (1 byte)
    def test_ivrs_parser_dte_00h(self):
        # Reserved device
        dte_t_00h = bytes([0x00, 0x00, 0x00, 0x00])
        IvrsParserTest.dte_00h = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_00h)
        self.assertNotEqual(IvrsParserTest.dte_00h.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_00h.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.RESERVED)
        self.assertEqual(IvrsParserTest.dte_00h.DeviceID, 0)
        self.assertEqual(IvrsParserTest.dte_00h.DTESetting, 0)

    def test_ivrs_parser_dte_01h(self):
        # All devices
        dte_t_01h = bytes([0x01, 0xFF, 0xFF, 0x00])
        IvrsParserTest.dte_01h = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_01h)
        self.assertEqual(IvrsParserTest.dte_01h.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.ALL)
        self.assertEqual(IvrsParserTest.dte_01h.DTESetting, 0)

    def test_ivrs_parser_dte_02h(self):
        # Select device
        dte_t_02h = bytes([0x02, 0x5A, 0x5A, 0x00])
        IvrsParserTest.dte_02h = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_02h)
        self.assertNotEqual(IvrsParserTest.dte_02h.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_02h.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.SELECT)
        self.assertEqual(IvrsParserTest.dte_02h.DeviceID, 0x5A5A)
        self.assertEqual(IvrsParserTest.dte_02h.DTESetting, 0)

    def test_ivrs_parser_dte_03h(self):
        # Start of device range
        dte_t_03h = bytes([0x03, 0xBE, 0xBA, 0x00])
        dte_t_04h = bytes([0x04, 0xFF, 0xFF, 0x00])
        IvrsParserTest.dte_03h = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_03h + dte_t_04h)
        self.assertNotEqual(IvrsParserTest.dte_03h.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_03h.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.RANGE_START)
        self.assertEqual(IvrsParserTest.dte_03h.DeviceID, 0xBABE)
        self.assertEqual(IvrsParserTest.dte_03h.DTESetting, 0)
        self.assertEqual(IvrsParserTest.dte_03h.EndDeviceID, 0xFFFF)

    def test_ivrs_parser_dte_42h(self):
        # Alias select device
        dte_t_42h = bytes([0x42, 0xAD, 0xDE, 0x00, 0x00, 0xEF, 0xBE, 0x00])
        IvrsParserTest.dte_42h = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_42h)
        self.assertNotEqual(IvrsParserTest.dte_42h.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_42h.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.ALIAS_SELECT)
        self.assertEqual(IvrsParserTest.dte_42h.DeviceID, 0xDEAD)
        self.assertEqual(IvrsParserTest.dte_42h.DTESetting, 0)
        self.assertEqual(IvrsParserTest.dte_42h.SourceDeviceID, 0xBEEF)

    def test_ivrs_parser_dte_43h(self):
        # Alias range device
        dte_t_43h = bytes([0x43, 0xED, 0xFE, 0x00, 0x00, 0x0D, 0xF0, 0x00])
        dte_t_04h = bytes([0x04, 0xFF, 0xFF, 0x00])
        IvrsParserTest.dte_43h = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_43h + dte_t_04h)
        self.assertNotEqual(IvrsParserTest.dte_43h.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_43h.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.ALIAS_RANGE_START)
        self.assertEqual(IvrsParserTest.dte_43h.DeviceID, 0xFEED)
        self.assertEqual(IvrsParserTest.dte_43h.DTESetting, 0)
        self.assertEqual(IvrsParserTest.dte_43h.SourceDeviceID, 0xF00D)
        self.assertEqual(IvrsParserTest.dte_43h.EndDeviceID, 0xFFFF)

    def test_ivrs_parser_dte_46h(self):
        # Extended select device
        dte_t_46h = bytes([0x46, 0x05, 0xB1, 0x00, 0xFE, 0xCA, 0xEF, 0xBE])
        IvrsParserTest.dte_46h = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_46h)
        self.assertNotEqual(IvrsParserTest.dte_46h.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_46h.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.EX_SELECT)
        self.assertEqual(IvrsParserTest.dte_46h.DeviceID, 0xB105)
        self.assertEqual(IvrsParserTest.dte_46h.DTESetting, 0)
        self.assertEqual(IvrsParserTest.dte_46h.ExtendedDTESetting, 0xBEEFCAFE)

    def test_ivrs_parser_dte_47h(self):
        # Extended range of device
        dte_t_47h = bytes([0x47, 0xDE, 0xC0, 0x00, 0xBE, 0xBA, 0xAD, 0xAB])
        dte_t_04h = bytes([0x04, 0xFF, 0xFF, 0x00])
        IvrsParserTest.dte_47h = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_47h + dte_t_04h)
        self.assertNotEqual(IvrsParserTest.dte_47h.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_47h.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.EX_RANGE_START)
        self.assertEqual(IvrsParserTest.dte_47h.DeviceID, 0xC0DE)
        self.assertEqual(IvrsParserTest.dte_47h.DTESetting, 0)
        self.assertEqual(IvrsParserTest.dte_47h.ExtendedDTESetting, 0xABADBABE)
        self.assertEqual(IvrsParserTest.dte_47h.EndDeviceID, 0xFFFF)

    def test_ivrs_parser_dte_48h(self):
        # Special device (IOAPIC, HPET)
        dte_t_48h = bytes([0x48, 0x00, 0x00, 0x00, 0x15, 0xAD, 0xDE, 0x01])
        IvrsParserTest.dte_48h = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_48h)
        self.assertNotEqual(IvrsParserTest.dte_48h.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_48h.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.SPECIAL)
        self.assertEqual(IvrsParserTest.dte_48h.DTESetting, 0)
        self.assertEqual(IvrsParserTest.dte_48h.Handle, 0x15)
        self.assertEqual(IvrsParserTest.dte_48h.SourceDeviceID, 0xDEAD)
        self.assertEqual(IvrsParserTest.dte_48h.Variety, 0x01)

    def test_ivrs_parser_dte_f0h_0(self):
        # ACPI device without UID
        dte_t_f0h_0 = bytes([0xF0, 0x11, 0x11, 0xF6,
                            0x46, 0x41, 0x4B, 0x45, 0x30, 0x30, 0x30, 0x30,     # HID: 'FAKE0000'
                            0x43, 0x4F, 0x4D, 0x50, 0x30, 0x30, 0x30, 0x30,     # CID: 'COMP0000'
                            0x00,   # UID format
                            0x00])  # UID length
        IvrsParserTest.dte_f0h_0 = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_f0h_0)
        self.assertNotEqual(IvrsParserTest.dte_f0h_0.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_f0h_0.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.ACPI)
        self.assertEqual(IvrsParserTest.dte_f0h_0.DeviceID, 0x1111)
        self.assertEqual(IvrsParserTest.dte_f0h_0.DTESetting, 0xF6)
        self.assertEqual(IvrsParserTest.dte_f0h_0.HID, b'FAKE0000')
        self.assertEqual(IvrsParserTest.dte_f0h_0.CID, b'COMP0000')
        self.assertEqual(IvrsParserTest.dte_f0h_0.UIDFormat, 0x00)
        self.assertEqual(IvrsParserTest.dte_f0h_0.UIDLength, 0x00)

    def test_ivrs_parser_dte_f0h_1(self):
        # ACPI device with integer UID
        dte_t_f0h_1 = bytes([0xF0, 0x11, 0x11, 0xF6,
                            0x46, 0x41, 0x4B, 0x45, 0x30, 0x30, 0x30, 0x30,     # HID: 'FAKE0000'
                            0x43, 0x4F, 0x4D, 0x50, 0x30, 0x30, 0x30, 0x30,     # CID: 'COMP0000'
                            0x01,   # UID format
                            0x08,   # UID length
                            0x0D, 0xF0, 0xED, 0xFE, 0xEF, 0xBE, 0xAD, 0xDE])    # UID: 0xDEADBEEFFEEDF00D
        IvrsParserTest.dte_f0h_1 = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_f0h_1)
        self.assertNotEqual(IvrsParserTest.dte_f0h_1.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_f0h_1.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.ACPI)
        self.assertEqual(IvrsParserTest.dte_f0h_1.DeviceID, 0x1111)
        self.assertEqual(IvrsParserTest.dte_f0h_1.DTESetting, 0xF6)
        self.assertEqual(IvrsParserTest.dte_f0h_1.HID, b'FAKE0000')
        self.assertEqual(IvrsParserTest.dte_f0h_1.CID, b'COMP0000')
        self.assertEqual(IvrsParserTest.dte_f0h_1.UIDFormat, 0x01)
        self.assertEqual(IvrsParserTest.dte_f0h_1.UIDLength, 0x08)
        self.assertEqual(IvrsParserTest.dte_f0h_1.UID, 0xDEADBEEFFEEDF00D)

    def test_ivrs_parser_dte_f0h_2(self):
        # ACPI device with string UID
        dte_t_f0h_2 = bytes([0xF0, 0x11, 0x11, 0xF6,
                            0x46, 0x41, 0x4B, 0x45, 0x30, 0x30, 0x30, 0x30,     # HID: 'FAKE0000'
                            0x43, 0x4F, 0x4D, 0x50, 0x30, 0x30, 0x30, 0x30,     # CID: 'COMP0000'
                            0x02,   # UID format
                            0x09,   # UID length
                            0x5C, 0x5F, 0x53, 0x42, 0x2E, 0x46, 0x55, 0x52, 0x30])  # UID: '\_SB.FUR0'
        IvrsParserTest.dte_f0h_2 = IVRS_TABLE.DEVICE_TABLE_ENTRY.Factory(dte_t_f0h_2)
        self.assertNotEqual(IvrsParserTest.dte_f0h_2.Encode(), None)
        self.assertEqual(IvrsParserTest.dte_f0h_2.Type, IVRS_TABLE.DEVICE_TABLE_ENTRY.DTE_TYPE.ACPI)
        self.assertEqual(IvrsParserTest.dte_f0h_2.DeviceID, 0x1111)
        self.assertEqual(IvrsParserTest.dte_f0h_2.DTESetting, 0xF6)
        self.assertEqual(IvrsParserTest.dte_f0h_2.HID, b'FAKE0000')
        self.assertEqual(IvrsParserTest.dte_f0h_2.CID, b'COMP0000')
        self.assertEqual(IvrsParserTest.dte_f0h_2.UIDFormat, 0x02)
        self.assertEqual(IvrsParserTest.dte_f0h_2.UIDLength, 0x09)
        self.assertEqual(IvrsParserTest.dte_f0h_2.UID, b'\\_SB.FUR0')

    def test_ivrs_parser_ivhd_10h(self):
        # I/O Virtualization Hardware Definition (IVHD) Type 10h header
        ivhd_t_10h = bytes([0x10,           # Type
                            0x90,           # Flags
                            0x18, 0x00,     # Length
                            0x02, 0x00,     # DeviceID
                            0x40, 0x00,     # Capability offset
                            0xEF, 0xBE, 0xAD, 0xDE, 0x0D, 0xF0, 0xED, 0xFE,     # IOMMU base address
                            0x00, 0x00,     # PCI Segment Group
                            0x00, 0x00,     # IOMMU info
                            0xBE, 0xBA, 0xAD, 0xAB])    # IOMMU Feature Reporting

        IvrsParserTest.ivhd_10h = IVRS_TABLE.IVHD_STRUCT(ivhd_t_10h)
        self.assertNotEqual(IvrsParserTest.ivhd_10h.Encode(), None)
        self.assertEqual(IvrsParserTest.ivhd_10h.Type, IVRS_TABLE.IVHD_STRUCT.IVHD_TYPE.TYPE_10H)
        self.assertEqual(IvrsParserTest.ivhd_10h.Flags, 0x90)
        self.assertEqual(IvrsParserTest.ivhd_10h.Length, 0x18)
        self.assertEqual(IvrsParserTest.ivhd_10h.DeviceID, 0x02)
        self.assertEqual(IvrsParserTest.ivhd_10h.CapabilityOffset, 0x40)
        self.assertEqual(IvrsParserTest.ivhd_10h.IOMMUBaseAddress, 0xFEEDF00DDEADBEEF)
        self.assertEqual(IvrsParserTest.ivhd_10h.SegmentGroup, 0)
        self.assertEqual(IvrsParserTest.ivhd_10h.IOMMUInfo, 0)
        self.assertEqual(IvrsParserTest.ivhd_10h.IOMMUFeatureInfo, 0xABADBABE)

    def test_ivrs_parser_ivhd_11h(self):
        # I/O Virtualization Hardware Definition (IVHD) Type 11h header
        ivhd_t_11h = bytes([0x11,           # Type
                            0x90,           # Flags
                            0x28, 0x00,     # Length
                            0x02, 0x00,     # DeviceID
                            0x40, 0x00,     # Capability offset
                            0xEF, 0xBE, 0xAD, 0xDE, 0x0D, 0xF0, 0xED, 0xFE,     # IOMMU base address
                            0x00, 0x00,     # PCI Segment Group
                            0x00, 0x00,     # IOMMU info
                            0xBE, 0xBA, 0xAD, 0xAB,    # IOMMU Attributes
                            0xDA, 0x4A, 0x29, 0x22, 0xEF, 0x77, 0x4F, 0x00,     # EFR Register Image
                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00])    # Reserved

        IvrsParserTest.ivhd_11h = IVRS_TABLE.IVHD_STRUCT(ivhd_t_11h)
        self.assertNotEqual(IvrsParserTest.ivhd_11h.Encode(), None)
        self.assertEqual(IvrsParserTest.ivhd_11h.Type, IVRS_TABLE.IVHD_STRUCT.IVHD_TYPE.TYPE_11H)
        self.assertEqual(IvrsParserTest.ivhd_11h.Flags, 0x90)
        self.assertEqual(IvrsParserTest.ivhd_11h.Length, 0x28)
        self.assertEqual(IvrsParserTest.ivhd_11h.DeviceID, 0x02)
        self.assertEqual(IvrsParserTest.ivhd_11h.CapabilityOffset, 0x40)
        self.assertEqual(IvrsParserTest.ivhd_11h.IOMMUBaseAddress, 0xFEEDF00DDEADBEEF)
        self.assertEqual(IvrsParserTest.ivhd_11h.SegmentGroup, 0)
        self.assertEqual(IvrsParserTest.ivhd_11h.IOMMUInfo, 0)
        self.assertEqual(IvrsParserTest.ivhd_11h.IOMMUFeatureInfo, 0xABADBABE)
        self.assertEqual(IvrsParserTest.ivhd_11h.IOMMUEFRImage, 0x4F77EF22294ADA)

    def test_ivrs_parser_ivhd_40h(self):
        # I/O Virtualization Hardware Definition (IVHD) Type 40h header
        ivhd_t_40h = bytes([0x40,           # Type
                            0x90,           # Flags
                            0x28, 0x00,     # Length
                            0x02, 0x00,     # DeviceID
                            0x40, 0x00,     # Capability offset
                            0xEF, 0xBE, 0xAD, 0xDE, 0x0D, 0xF0, 0xED, 0xFE,     # IOMMU base address
                            0x00, 0x00,     # PCI Segment Group
                            0x00, 0x00,     # IOMMU info
                            0xBE, 0xBA, 0xAD, 0xAB,    # IOMMU Attributes
                            0xDA, 0x4A, 0x29, 0x22, 0xEF, 0x77, 0x4F, 0x00,     # EFR Register Image
                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00])    # Reserved

        IvrsParserTest.ivhd_40h = IVRS_TABLE.IVHD_STRUCT(ivhd_t_40h)
        self.assertNotEqual(IvrsParserTest.ivhd_40h.Encode(), None)
        self.assertEqual(IvrsParserTest.ivhd_40h.Type, IVRS_TABLE.IVHD_STRUCT.IVHD_TYPE.TYPE_40H)
        self.assertEqual(IvrsParserTest.ivhd_40h.Flags, 0x90)
        self.assertEqual(IvrsParserTest.ivhd_40h.Length, 0x28)
        self.assertEqual(IvrsParserTest.ivhd_40h.DeviceID, 0x02)
        self.assertEqual(IvrsParserTest.ivhd_40h.CapabilityOffset, 0x40)
        self.assertEqual(IvrsParserTest.ivhd_40h.IOMMUBaseAddress, 0xFEEDF00DDEADBEEF)
        self.assertEqual(IvrsParserTest.ivhd_40h.SegmentGroup, 0)
        self.assertEqual(IvrsParserTest.ivhd_40h.IOMMUInfo, 0)
        self.assertEqual(IvrsParserTest.ivhd_40h.IOMMUFeatureInfo, 0xABADBABE)
        self.assertEqual(IvrsParserTest.ivhd_40h.IOMMUEFRImage, 0x4F77EF22294ADA)

    def test_ivrs_parser_ivmd_20h(self):
        # IVMD Types 20h Format
        ivmd_t_20h = bytes([0x20,           # Type
                            0x08,           # Flags
                            0x20, 0x00,     # Length
                            0x00, 0x00,     # DeviceID
                            0x00, 0x00,     # Auxiliary data
                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,     # Reserved
                            0xEF, 0xBE, 0xAD, 0xDE, 0x0D, 0xF0, 0xED, 0xFE,     # IVMD start address
                            0x00, 0xAD, 0xBB, 0xDA, 0xDE, 0xC0, 0x05, 0xB1])    # IVMD memory block length

        IvrsParserTest.ivmd_20h = IVRS_TABLE.IVMD_STRUCT(ivmd_t_20h)
        self.assertNotEqual(IvrsParserTest.ivmd_20h.Encode(), None)
        self.assertEqual(IvrsParserTest.ivmd_20h.Type, IVRS_TABLE.IVMD_STRUCT.IVMD_TYPE.TYPE_20H)
        self.assertEqual(IvrsParserTest.ivmd_20h.Flags, 0x08)
        self.assertEqual(IvrsParserTest.ivmd_20h.Length, 0x20)
        self.assertEqual(IvrsParserTest.ivmd_20h.DeviceID, 0)
        self.assertEqual(IvrsParserTest.ivmd_20h.AuxiliaryData, 0)
        self.assertEqual(IvrsParserTest.ivmd_20h.Reserved, 0)
        self.assertEqual(IvrsParserTest.ivmd_20h.IVMDStartAddress, 0xFEEDF00DDEADBEEF)
        self.assertEqual(IvrsParserTest.ivmd_20h.IVMDMemoryBlockLength, 0xB105C0DEDABBAD00)

    def test_ivrs_parser_ivmd_21h(self):
        # IVMD Types 21h Format
        ivmd_t_21h = bytes([0x21,           # Type
                            0x08,           # Flags
                            0x20, 0x00,     # Length
                            0x00, 0x00,     # DeviceID
                            0x00, 0x00,     # Auxiliary data
                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,     # Reserved
                            0xEF, 0xBE, 0xAD, 0xDE, 0x0D, 0xF0, 0xED, 0xFE,     # IVMD start address
                            0x00, 0xAD, 0xBB, 0xDA, 0xDE, 0xC0, 0x05, 0xB1])    # IVMD memory block length

        IvrsParserTest.ivmd_21h = IVRS_TABLE.IVMD_STRUCT(ivmd_t_21h)
        self.assertNotEqual(IvrsParserTest.ivmd_21h.Encode(), None)
        self.assertEqual(IvrsParserTest.ivmd_21h.Type, IVRS_TABLE.IVMD_STRUCT.IVMD_TYPE.TYPE_21H)
        self.assertEqual(IvrsParserTest.ivmd_21h.Flags, 0x08)
        self.assertEqual(IvrsParserTest.ivmd_21h.Length, 0x20)
        self.assertEqual(IvrsParserTest.ivmd_21h.DeviceID, 0)
        self.assertEqual(IvrsParserTest.ivmd_21h.AuxiliaryData, 0)
        self.assertEqual(IvrsParserTest.ivmd_21h.Reserved, 0)
        self.assertEqual(IvrsParserTest.ivmd_21h.IVMDStartAddress, 0xFEEDF00DDEADBEEF)
        self.assertEqual(IvrsParserTest.ivmd_21h.IVMDMemoryBlockLength, 0xB105C0DEDABBAD00)

    def test_ivrs_parser_ivmd_22h(self):
        # IVMD Types 22h Format
        ivmd_t_22h = bytes([0x22,           # Type
                            0x08,           # Flags
                            0x20, 0x00,     # Length
                            0x00, 0x00,     # DeviceID
                            0x00, 0x00,     # Auxiliary data
                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,     # Reserved
                            0xEF, 0xBE, 0xAD, 0xDE, 0x0D, 0xF0, 0xED, 0xFE,     # IVMD start address
                            0x00, 0xAD, 0xBB, 0xDA, 0xDE, 0xC0, 0x05, 0xB1])    # IVMD memory block length

        IvrsParserTest.ivmd_22h = IVRS_TABLE.IVMD_STRUCT(ivmd_t_22h)
        self.assertNotEqual(IvrsParserTest.ivmd_22h.Encode(), None)
        self.assertEqual(IvrsParserTest.ivmd_22h.Type, IVRS_TABLE.IVMD_STRUCT.IVMD_TYPE.TYPE_22H)
        self.assertEqual(IvrsParserTest.ivmd_22h.Flags, 0x08)
        self.assertEqual(IvrsParserTest.ivmd_22h.Length, 0x20)
        self.assertEqual(IvrsParserTest.ivmd_22h.DeviceID, 0)
        self.assertEqual(IvrsParserTest.ivmd_22h.AuxiliaryData, 0)
        self.assertEqual(IvrsParserTest.ivmd_22h.Reserved, 0)
        self.assertEqual(IvrsParserTest.ivmd_22h.IVMDStartAddress, 0xFEEDF00DDEADBEEF)
        self.assertEqual(IvrsParserTest.ivmd_22h.IVMDMemoryBlockLength, 0xB105C0DEDABBAD00)

    def test_ivrs_parser_ivrs_empty(self):
        ivrs_header = bytes([0x49, 0x56, 0x52, 0x53,    # Signature: IVRS
                            0x30, 0x00, 0x00, 0x00,     # Length
                            0x02,                       # Revision
                            0x9C,                       # Checksum
                            0x41, 0x4D, 0x44, 0x20, 0x20, 0x20,     # OEM ID: 'AMD  '
                            0x41, 0x4D, 0x44, 0x20, 0x49, 0x56, 0x52, 0x53,     # OEM Table ID: 'AMD IVRS'
                            0x01, 0x00, 0x00, 0x00,     # OEM Revision
                            0x41, 0x4D, 0x44, 0x20,     # CreatorID: 'AMD '
                            0x00, 0x00, 0x00, 0x00,     # Creator revision
                            0x43, 0x30, 0x20, 0x00,     # IVinfo
                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00])    # Reserved: 0

        IvrsParserTest.ivrs = IVRS_TABLE(ivrs_header)
        self.assertNotEqual(IvrsParserTest.ivrs.Encode(), None)

        self.assertEqual(IvrsParserTest.ivrs.acpi_header.Signature, b'IVRS')
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.Length, 0x30)
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.Revision, 0x02)
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.Checksum, 0x9C)
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.OEMID, b'AMD   ')
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.OEMTableID, b'AMD IVRS')
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.OEMRevision, 1)
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.CreatorID, b'AMD ')
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.CreatorRevision, 0)
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.IVinfo, 0x00203043)
        self.assertEqual(IvrsParserTest.ivrs.acpi_header.Reserved, 0)

    def test_ivrs_parser_ivrs_full(self):
        # finally the big daddy... load it up!
        IvrsParserTest.ivhd_10h.addDTEEntry(IvrsParserTest.dte_01h)
        IvrsParserTest.ivhd_10h.addDTEEntry(IvrsParserTest.dte_02h)
        IvrsParserTest.ivhd_10h.addDTEEntry(IvrsParserTest.dte_03h)
        IvrsParserTest.ivhd_10h.addDTEEntry(IvrsParserTest.dte_00h)

        IvrsParserTest.ivhd_11h.addDTEEntry(IvrsParserTest.dte_42h)
        IvrsParserTest.ivhd_11h.addDTEEntry(IvrsParserTest.dte_43h)
        IvrsParserTest.ivhd_11h.addDTEEntry(IvrsParserTest.dte_46h)
        IvrsParserTest.ivhd_11h.addDTEEntry(IvrsParserTest.dte_47h)
        IvrsParserTest.ivhd_11h.addDTEEntry(IvrsParserTest.dte_48h)

        IvrsParserTest.ivhd_40h.addDTEEntry(IvrsParserTest.dte_f0h_0)
        IvrsParserTest.ivhd_40h.addDTEEntry(IvrsParserTest.dte_f0h_1)
        IvrsParserTest.ivhd_40h.addDTEEntry(IvrsParserTest.dte_f0h_2)

        IvrsParserTest.ivrs.addIVHDEntry(IvrsParserTest.ivhd_10h)
        IvrsParserTest.ivrs.addIVMDEntry(IvrsParserTest.ivmd_20h)
        IvrsParserTest.ivrs.addIVHDEntry(IvrsParserTest.ivhd_11h)
        IvrsParserTest.ivrs.addIVMDEntry(IvrsParserTest.ivmd_21h)
        IvrsParserTest.ivrs.addIVHDEntry(IvrsParserTest.ivhd_40h)
        IvrsParserTest.ivrs.addIVMDEntry(IvrsParserTest.ivmd_22h)

        IvrsParserTest.ivrs.DumpInfo()
        IvrsParserTest.ivrs.ToXmlElementTree()

        ivrs_byte = IvrsParserTest.ivrs.Encode()
        ivrs2 = IVRS_TABLE(ivrs_byte)
        self.assertEqual(ivrs2.Encode(), ivrs_byte)


if __name__ == '__main__':
    unittest.main()



# @file path_utilities_test.py
# Contains unit test routines for the path_utilities class.
#
#
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##

import unittest
import os
import sys
import tempfile
import shutil
from edk2toollib.uefi.edk2.path_utilities import Edk2Path


class PathUtilitiesTest(unittest.TestCase):

    def _make_file_helper(self, parentpath: str, filename: str) -> None:
        ''' simple internal helper to make a file and write basic content'''
        with open(os.path.join(parentpath, filename), "w") as f:
            f.write("Unit test content")

    def _make_edk2_module_helper(self, parentpath: str, modname: str, extension_case_lower: bool = True) -> str:
        ''' simple internal helper to make a folder and inf like edk2
        return the path to the module folder
        '''
        modfolder = os.path.join(parentpath, modname)
        os.makedirs(modfolder, exist_ok=True)
        fn = modname + (".inf" if extension_case_lower else ".INF")
        self._make_file_helper(modfolder, fn)
        return modfolder

    def _make_edk2_package_helper(self, path: str, packagename: str, extension_case_lower: bool = True) -> str:
        ''' simple internal helper to make an ed2 package as follows:

        path/
          packagename/
            packagename.dec
            module1/
              module1.inf
            module2/
              module2.inf
              X64/
                TestFile.c

        return the path to the new package
        '''
        pkgfolder = os.path.join(path, packagename)
        os.makedirs(pkgfolder, exist_ok=True)
        pn = packagename + (".dec" if extension_case_lower else ".DEC")
        self._make_file_helper(pkgfolder, pn)
        self._make_edk2_module_helper(pkgfolder, "module1", extension_case_lower=extension_case_lower)
        p2 = self._make_edk2_module_helper(pkgfolder, "module2", extension_case_lower=extension_case_lower)
        p3 = os.path.join(p2, "X64")
        os.makedirs(p3)
        self._make_file_helper(p3, "TestFile.c")
        return pkgfolder

    def setUp(self):
        ''' unittest fixture run before each test.
        Create a tmpdir and set the current working dir there'''
        self.tmp = tempfile.mkdtemp()
        self.precwd = os.getcwd()
        os.chdir(self.tmp)  # move to tempfile root

    def tearDown(self):
        ''' unittest fixture run after each test.
        Delete the tempdir and set the current working dir back'''
        os.chdir(self.precwd)
        if os.path.isdir(self.tmp):
            shutil.rmtree(self.tmp)

    # TESTS

    def test_basic_init_ws_abs(self):
        ''' test edk2path with valid absolute path to workspace'''
        pathobj = Edk2Path(self.tmp, [])
        self.assertEqual(pathobj.WorkspacePath, self.tmp)

    def test_basic_init_ws_cwd(self):
        ''' test edk2path with a relative path to workspace'''
        relpath = "testrootfolder"
        fullpath = os.path.join(self.tmp, relpath)
        os.mkdir(fullpath)
        pathobj = Edk2Path(relpath, [])
        self.assertEqual(pathobj.WorkspacePath, fullpath)

    def test_nonexistant_ws(self):
        ''' test edk2path with invalid workspace'''

        invalid_ws = os.path.join(self.tmp, "invalidpath")
        with self.assertRaises(Exception):
            Edk2Path(invalid_ws, [])

    def test_nonexistant_abs(self):
        ''' test edk2path with valid ws but invalid pp'''
        pp = "doesnot_exist"
        pp_full_1 = os.path.join(self.tmp, pp)
        # absolute path
        with self.assertRaises(Exception):
            Edk2Path(self.tmp, [pp_full_1])
        # relative path
        with self.assertRaises(Exception):
            Edk2Path(self.tmp, [pp])
        # confirm optional parameter to remove invalid pp values
        pathobj = Edk2Path(self.tmp, [pp], error_on_invalid_pp=False)
        self.assertEqual(len(pathobj.PackagePathList), 0)

    def test_pp_inside_workspace(self):
        ''' test with packagespath pointing to folder nested inside workspace
        root/                   <-- current working directory
            folder_ws/           <-- workspace root
                folder_pp/       <-- packages path
                    pp packages here
                ws packages here
        '''
        ws_rel = "folder_ws"
        ws_abs = os.path.join(self.tmp, ws_rel)
        os.mkdir(ws_abs)
        folder_pp_rel = "pp1"
        folder_pp1_abs = os.path.join(ws_abs, folder_pp_rel)
        os.mkdir(folder_pp1_abs)

        # pp absolute
        pathobj = Edk2Path(ws_abs, [folder_pp1_abs])
        self.assertEqual(pathobj.WorkspacePath, ws_abs)
        self.assertEqual(pathobj.PackagePathList[0], folder_pp1_abs)

        # pp relative to workspace
        pathobj = Edk2Path(ws_abs, [folder_pp_rel])
        self.assertEqual(pathobj.WorkspacePath, ws_abs)
        self.assertEqual(pathobj.PackagePathList[0], folder_pp1_abs)

        # pp relative to cwd
        pathobj = Edk2Path(ws_abs, [os.path.join(ws_rel, folder_pp_rel)])
        self.assertEqual(pathobj.WorkspacePath, ws_abs)
        self.assertEqual(pathobj.PackagePathList[0], folder_pp1_abs)

    def test_pp_outside_workspace(self):
        ''' test with packagespath pointing to folder outside of workspace
        root/                   <-- current working directory
            folder_ws/           <-- workspace root
                ws packages here
            folder_pp/       <-- packages path
                pp packages here

        '''
        ws_rel = "folder_ws"
        ws_abs = os.path.join(self.tmp, ws_rel)
        os.mkdir(ws_abs)
        folder_pp_rel = "pp1"
        folder_pp1_abs = os.path.join(self.tmp, folder_pp_rel)
        os.mkdir(folder_pp1_abs)

        # pp absolute
        pathobj = Edk2Path(ws_abs, [folder_pp1_abs])
        self.assertEqual(pathobj.WorkspacePath, ws_abs)
        self.assertEqual(pathobj.PackagePathList[0], folder_pp1_abs)

        # pp relative to cwd
        pathobj = Edk2Path(ws_abs, [folder_pp_rel])
        self.assertEqual(pathobj.WorkspacePath, ws_abs)
        self.assertEqual(pathobj.PackagePathList[0], folder_pp1_abs)

    @unittest.skipUnless(sys.platform.startswith("win"), "requires Windows")
    def test_basic_init_ws_abs_different_case(self):
        inputPath = self.tmp.capitalize()
        if self.tmp[0].isupper():
            inputPath = self.tmp[0].lower() + self.tmp[1:]

        pathobj = Edk2Path(inputPath, [])
        self.assertNotEqual(pathobj.WorkspacePath, self.tmp)

    def test_get_containing_package_inside_workspace(self):
        ''' test basic usage of GetContainingPackage with packages path nested
        inside the workspace

        File layout:

         root/                  <-- current working directory (self.tmp)
            folder_ws/           <-- workspace root
                folder_pp/       <-- packages path
                    PPTestPkg/   <-- A edk2 package
                        PPTestPkg.DEC
                        module1/
                            module1.INF
                        module2/
                            module2.INF
                            X64/
                                TestFile.c
                WSTestPkg/   <-- A edk2 package
                    WSTestPkg.dec
                    module1/
                        module1.inf
                    module2/
                        module2.inf
                        X64/
                            TestFile.c
        '''
        ws_rel = "folder_ws"
        ws_abs = os.path.join(self.tmp, ws_rel)
        os.mkdir(ws_abs)
        folder_pp_rel = "pp1"
        folder_pp1_abs = os.path.join(ws_abs, folder_pp_rel)
        os.mkdir(folder_pp1_abs)
        ws_p_name = "WSTestPkg"
        ws_pkg_abs = self._make_edk2_package_helper(ws_abs, ws_p_name)
        pp_p_name = "PPTestPkg"
        pp_pkg_abs = self._make_edk2_package_helper(folder_pp1_abs, pp_p_name, extension_case_lower=False)
        pathobj = Edk2Path(ws_abs, [folder_pp1_abs])

        # file in WSTestPkg root
        p = os.path.join(ws_pkg_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), ws_p_name)

        # file in module in WSTestPkg
        p = os.path.join(ws_pkg_abs, "module1", "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), ws_p_name)

        # file in workspace root - no package- should return ws root
        p = os.path.join(ws_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), ws_rel)

        # file outside of the workspace - invalid and should return None
        p = os.path.join(os.path.dirname(ws_abs), "testfile.c")
        self.assertIsNone(pathobj.GetContainingPackage(p))

        # file in PPTestPkg root
        p = os.path.join(pp_pkg_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), pp_p_name)

        # file in module in WSTestPkg
        p = os.path.join(pp_pkg_abs, "module1", "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), pp_p_name)

        # file in packages path root - no package- should return packages path dir
        p = os.path.join(folder_pp1_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), folder_pp_rel)

    def test_get_containing_package_outside_workspace(self):
        ''' test basic usage of GetContainingPackage with packages path
        outside the workspace

        File layout:

         root/                  <-- current working directory (self.tmp)
            folder_ws/           <-- workspace root
                WSTestPkg/   <-- A edk2 package
                    WSTestPkg.dec
                    module1/
                        module1.inf
                    module2/
                        module2.inf
                        X64/
                            TestFile.c
            folder_pp/       <-- packages path
                PPTestPkg/   <-- A edk2 package
                    PPTestPkg.DEC
                    module1/
                        module1.INF
                    module2/
                        module2.INF
                        X64/
                            TestFile.c
        '''
        ws_rel = "folder_ws"
        ws_abs = os.path.join(self.tmp, ws_rel)
        os.mkdir(ws_abs)
        folder_pp_rel = "pp1"
        folder_pp1_abs = os.path.join(self.tmp, folder_pp_rel)
        os.mkdir(folder_pp1_abs)
        ws_p_name = "WSTestPkg"
        ws_pkg_abs = self._make_edk2_package_helper(ws_abs, ws_p_name)
        pp_p_name = "PPTestPkg"
        pp_pkg_abs = self._make_edk2_package_helper(folder_pp1_abs, pp_p_name, extension_case_lower=False)
        pathobj = Edk2Path(ws_abs, [folder_pp1_abs])

        # file in WSTestPkg root
        p = os.path.join(ws_pkg_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), ws_p_name)

        # file in module in WSTestPkg
        p = os.path.join(ws_pkg_abs, "module1", "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), ws_p_name)

        # file in workspace root - no package- should return ws root
        p = os.path.join(ws_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), ws_rel)

        # file outside of the workspace - invalid and should return None
        p = os.path.join(os.path.dirname(ws_abs), "testfile.c")
        self.assertIsNone(pathobj.GetContainingPackage(p))

        # file in PPTestPkg root
        p = os.path.join(pp_pkg_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), pp_p_name)

        # file in module in WSTestPkg
        p = os.path.join(pp_pkg_abs, "module1", "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), pp_p_name)

        # file in packages path root - no package- should return packages path dir
        p = os.path.join(folder_pp1_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), folder_pp_rel)

    @unittest.skipUnless(sys.platform.startswith("win"), "requires Windows")
    def test_get_containing_package_ws_abs_different_case(self):
        ''' test basic usage of GetContainingPackage when the workspace path has different case for
        the drive letter then the incoming paths. This can happen on Windows
        if os.path.realpath is used.

        File layout:

         root/                  <-- current working directory (self.tmp)
            folder_ws/           <-- workspace root
                folder_pp/       <-- packages path
                    PPTestPkg/   <-- A edk2 package
                        PPTestPkg.DEC
                        module1/
                            module1.INF
                        module2/
                            module2.INF
                            X64/
                                TestFile.c
                WSTestPkg/   <-- A edk2 package
                    WSTestPkg.dec
                    module1/
                        module1.inf
                    module2/
                        module2.inf
                        X64/
                            TestFile.c
        '''
        ws_rel = "folder_ws"
        ws_abs = os.path.join(self.tmp, ws_rel)
        wsi_abs = os.path.join(self.tmp, ws_rel.capitalize())  # invert the case of the first char of the ws folder name
        os.mkdir(ws_abs)
        folder_pp_rel = "pp1"
        folder_pp1_abs = os.path.join(ws_abs, folder_pp_rel)
        os.mkdir(folder_pp1_abs)
        ws_p_name = "WSTestPkg"
        ws_pkg_abs = self._make_edk2_package_helper(ws_abs, ws_p_name)
        pp_p_name = "PPTestPkg"
        pp_pkg_abs = self._make_edk2_package_helper(folder_pp1_abs, pp_p_name, extension_case_lower=False)
        pathobj = Edk2Path(wsi_abs, [folder_pp1_abs])

        # confirm inverted
        self.assertNotEqual(pathobj.WorkspacePath, ws_abs)
        self.assertEqual(os.path.normcase(pathobj.WorkspacePath), os.path.normcase(ws_abs))

        # file in WSTestPkg root
        p = os.path.join(ws_pkg_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), ws_p_name)

        # file in module in WSTestPkg
        p = os.path.join(ws_pkg_abs, "module1", "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), ws_p_name)

        # file in workspace root - no package- should return ws root
        p = os.path.join(ws_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), ws_rel)

        # file outside of the workspace - invalid and should return None
        p = os.path.join(os.path.dirname(ws_abs), "testfile.c")
        self.assertIsNone(pathobj.GetContainingPackage(p))

        # file in PPTestPkg root
        p = os.path.join(pp_pkg_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), pp_p_name)

        # file in module in WSTestPkg
        p = os.path.join(pp_pkg_abs, "module1", "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), pp_p_name)

        # file in packages path root - no package- should return packages path dir
        p = os.path.join(folder_pp1_abs, "testfile.c")
        self.assertEqual(pathobj.GetContainingPackage(p), folder_pp_rel)

    def test_get_containing_module(self):
        ''' test basic usage of GetContainingModule with packages path nested
        inside the workspace

        File layout:

         root/                  <-- current working directory (self.tmp)
            folder_ws/           <-- workspace root
                folder_pp/       <-- packages path
                    PPTestPkg/   <-- A edk2 package
                        PPTestPkg.DEC
                        module1/
                            module1.INF
                        module2/
                            module2.INF
                            X64/
                                TestFile.c
                WSTestPkg/   <-- A edk2 package
                    WSTestPkg.dec
                    module1/
                        module1.inf
                    module2/
                        module2.inf
                        X64/
                            TestFile.c
        '''
        ws_rel = "folder_ws"
        ws_abs = os.path.join(self.tmp, ws_rel)
        os.mkdir(ws_abs)
        folder_pp_rel = "pp1"
        folder_pp1_abs = os.path.join(ws_abs, folder_pp_rel)
        os.mkdir(folder_pp1_abs)
        ws_p_name = "WSTestPkg"
        ws_pkg_abs = self._make_edk2_package_helper(ws_abs, ws_p_name)
        pp_p_name = "PPTestPkg"
        pp_pkg_abs = self._make_edk2_package_helper(folder_pp1_abs, pp_p_name, extension_case_lower=False)
        pathobj = Edk2Path(ws_abs, [folder_pp1_abs])

        # file in WSTestPkg root.  Module list is empty
        p = os.path.join(ws_pkg_abs, "testfile.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 0)

        # file in module in WSTestPkg/module1/module1.inf
        p = os.path.join(ws_pkg_abs, "module1", "testfile.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 1)
        self.assertIn(os.path.join(ws_pkg_abs, "module1", "module1.inf"), relist)

        # file in workspace root - no package- should return ws root
        p = os.path.join(ws_abs, "testfile.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 0)

        # file outside of the workspace - invalid and should return None
        p = os.path.join(os.path.dirname(ws_abs), "testfile.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 0)

        # file in module2 x64
        p = os.path.join(ws_pkg_abs, "module2", "X64", "testfile.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 1)
        self.assertIn(os.path.join(ws_pkg_abs, "module2", "module2.inf"), relist)

        # inf file in module2 x64
        p = os.path.join(ws_pkg_abs, "module2", "module2.inf")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 1)
        self.assertIn(os.path.join(ws_pkg_abs, "module2", "module2.inf"), relist)

        # file in PPTestPkg root
        p = os.path.join(pp_pkg_abs, "testfile.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 0)

        # file in module in PPTestPkg
        p = os.path.join(pp_pkg_abs, "module1", "testfile.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 1)
        self.assertIn(os.path.join(pp_pkg_abs, "module1", "module1.INF"), relist)

        # inf file in module in PPTestPkg
        p = os.path.join(pp_pkg_abs, "module1", "module1.INF")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 1)
        self.assertIn(os.path.join(pp_pkg_abs, "module1", "module1.INF"), relist)

        # file in packages path root - no module
        p = os.path.join(folder_pp1_abs, "testfile.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 0)

        # file doesn't exist and parent folder doesn't exist
        p = os.path.join(ws_pkg_abs, "ThisParentDirDoesNotExist", "ThisFileDoesNotExist.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 0)

        # file doesn't exist and parent folder doesn't exist and parent parent folder doesn't exist
        p = os.path.join(ws_pkg_abs, "DirDirDoesNotExist", "DirDoesNotExist", "FileDoesNotExist.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 0)

        # file doesn't exist and parent folder doesn't exist but parent parent is valid module
        # file in module in WSTestPkg/module1/module1.inf
        p = os.path.join(ws_pkg_abs, "module1", "ThisParentDirDoesNotExist", "testfile.c")
        relist = pathobj.GetContainingModules(p)
        self.assertEqual(len(relist), 1)
        self.assertIn(os.path.join(ws_pkg_abs, "module1", "module1.inf"), relist)

    def test_get_edk2_relative_path_from_absolute_path(self):
        ''' test basic usage of GetEdk2RelativePathFromAbsolutePath with packages path nested
        inside the workspace

        File layout:

         root/                  <-- current working directory (self.tmp)
            folder_ws/           <-- workspace root
                folder_pp/       <-- packages path
                    PPTestPkg/   <-- A edk2 package
                        PPTestPkg.DEC
                        module1/
                            module1.INF
                        module2/
                            module2.INF
                            X64/
                                TestFile.c
                WSTestPkg/   <-- A edk2 package
                    WSTestPkg.dec
                    module1/
                        module1.inf
                    module2/
                        module2.inf
                        X64/
                            TestFile.c
        '''
        ws_rel = "folder_ws"
        ws_abs = os.path.join(self.tmp, ws_rel)
        os.mkdir(ws_abs)
        folder_pp_rel = "pp1"
        folder_pp1_abs = os.path.join(ws_abs, folder_pp_rel)
        os.mkdir(folder_pp1_abs)
        ws_p_name = "WSTestPkg"
        ws_pkg_abs = self._make_edk2_package_helper(ws_abs, ws_p_name)
        pp_p_name = "PPTestPkg"
        pp_pkg_abs = self._make_edk2_package_helper(folder_pp1_abs, pp_p_name, extension_case_lower=False)
        pathobj = Edk2Path(ws_abs, [folder_pp1_abs])

        # file in packages path
        p = os.path.join(pp_pkg_abs, "module1", "module1.INF")
        self.assertEqual(pathobj.GetEdk2RelativePathFromAbsolutePath(p), f"{pp_p_name}/module1/module1.INF")

        # file in workspace
        p = os.path.join(ws_pkg_abs, "module2", "X64", "TestFile.c")
        self.assertEqual(pathobj.GetEdk2RelativePathFromAbsolutePath(p), f"{ws_p_name}/module2/X64/TestFile.c")

        # file not in workspace
        p = os.path.join(self.tmp, "module2", "X64", "TestFile.c")
        self.assertIsNone(pathobj.GetEdk2RelativePathFromAbsolutePath(p))

        # pass in bad parameter
        self.assertIsNone(pathobj.GetEdk2RelativePathFromAbsolutePath(None))

        # file is outside of code tree and not absolute path
        p = os.path.join("module2", "X64", "TestFile.c")
        self.assertIsNone(pathobj.GetEdk2RelativePathFromAbsolutePath(p))

        # file is cwd relative but not absolute path
        p = os.path.join(ws_rel, ws_p_name, "module2", "X64", "TestFile.c")
        self.assertIsNone(pathobj.GetEdk2RelativePathFromAbsolutePath(p))

    def test_get_absolute_path_on_this_system_from_edk2_relative_path(self):
        ''' test basic usage of GetAbsolutePathOnThisSystemFromEdk2RelativePath with packages path nested
        inside the workspace

        File layout:

         root/                  <-- current working directory (self.tmp)
            folder_ws/           <-- workspace root
                folder_pp/       <-- packages path
                    PPTestPkg/   <-- A edk2 package
                        PPTestPkg.DEC
                        module1/
                            module1.INF
                        module2/
                            module2.INF
                            X64/
                                TestFile.c
                WSTestPkg/   <-- A edk2 package
                    WSTestPkg.dec
                    module1/
                        module1.inf
                    module2/
                        module2.inf
                        X64/
                            TestFile.c
        '''
        ws_rel = "folder_ws"
        ws_abs = os.path.join(self.tmp, ws_rel)
        os.mkdir(ws_abs)
        folder_pp_rel = "pp1"
        folder_pp1_abs = os.path.join(ws_abs, folder_pp_rel)
        os.mkdir(folder_pp1_abs)
        ws_p_name = "WSTestPkg"
        ws_pkg_abs = self._make_edk2_package_helper(ws_abs, ws_p_name)
        pp_p_name = "PPTestPkg"
        pp_pkg_abs = self._make_edk2_package_helper(folder_pp1_abs, pp_p_name, extension_case_lower=False)
        pathobj = Edk2Path(ws_abs, [folder_pp1_abs])

        # file in packages path
        ep = os.path.join(pp_pkg_abs, "module1", "module1.INF")
        rp = f"{pp_p_name}/module1/module1.INF"
        self.assertEqual(pathobj.GetAbsolutePathOnThisSystemFromEdk2RelativePath(rp), ep)

        # file in workspace
        ep = os.path.join(ws_pkg_abs, "module2", "X64", "TestFile.c")
        rp = f"{ws_p_name}/module2/X64/TestFile.c"
        self.assertEqual(pathobj.GetAbsolutePathOnThisSystemFromEdk2RelativePath(rp), ep)

        # file not in workspace
        rp = "DoesNotExistPkg/module2/X64/TestFile.c"
        self.assertIsNone(pathobj.GetAbsolutePathOnThisSystemFromEdk2RelativePath(rp))

        # pass in bad parameter
        self.assertIsNone(pathobj.GetAbsolutePathOnThisSystemFromEdk2RelativePath(None))



# @file
# Module contains helper classes and functions to work with Variable Policy structures
# and substructures.
#
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##

import uuid
import struct
from edk2toollib.uefi.uefi_multi_phase import EfiVariableAttributes


class VariableLockOnVarStatePolicy(object):
    # typedef struct {
    #     EFI_GUID Namespace;
    #     UINT8    Value;
    #     UINT8    Reserved;
    #     // CHAR16   Name[];           // Variable Length Field
    # } VARIABLE_LOCK_ON_VAR_STATE_POLICY;
    _HdrStructFormat = "<16sBB"
    _HdrStructSize = struct.calcsize(_HdrStructFormat)

    def __init__(self):
        self.Namespace = uuid.UUID(bytes=b'\x00' * 16)
        self.Value = 0
        self.Name = None

    def __str__(self):
        return "VARIABLE_LOCK_ON_VAR_STATE_POLICY(%s, %d, %s)" % (self.Namespace, self.Value, self.Name)

    def decode(self, buffer):
        """
        load this object from a bytes buffer

        return any remaining buffer
        """
        (_namespace, self.Value, _) = struct.unpack(
            self._HdrStructFormat, buffer[:self._HdrStructSize])

        self.Namespace = uuid.UUID(bytes_le=_namespace)

        # Scan the rest of the buffer for a \x00\x00 to terminate the string.
        buffer = buffer[self._HdrStructSize:]
        if len(buffer) < 4:
            raise ValueError("Buffer too short!")

        string_end = None
        for i in range(0, len(buffer), 2):
            if buffer[i] == 0 and buffer[i + 1] == 0:
                string_end = i + 2
                break

        if string_end is None:
            raise ValueError("String end not detected!")

        self.Name = buffer[:string_end].decode('utf-16').strip('\x00')

        return buffer[string_end:]


class VariablePolicyEntry(object):
    # typedef struct {
    #     UINT32   Version;
    #     UINT16   Size;
    #     UINT16   OffsetToName;
    #     EFI_GUID Namespace;
    #     UINT32   MinSize;
    #     UINT32   MaxSize;
    #     UINT32   AttributesMustHave;
    #     UINT32   AttributesCantHave;
    #     UINT8    LockPolicyType;
    #     UINT8    Reserved[3];
    #     // UINT8    LockPolicy[];     // Variable Length Field
    #     // CHAR16   Name[]            // Variable Length Field
    # } VARIABLE_POLICY_ENTRY;
    _HdrStructFormat = "<IHH16sIIIIB3s"             # spell-checker:disable-line
    _HdrStructSize = struct.calcsize(_HdrStructFormat)

    ENTRY_REVISION = 0x0001_0000

    NO_MIN_SIZE = 0
    NO_MAX_SIZE = 0xFFFF_FFFF
    NO_MUST_ATTR = 0
    NO_CANT_ATTR = 0

    TYPE_NO_LOCK = 0
    TYPE_LOCK_NOW = 1
    TYPE_LOCK_ON_CREATE = 2
    TYPE_LOCK_ON_VAR_STATE = 3

    LOCK_POLICY_STRING_MAP = {
        TYPE_NO_LOCK: "NONE",
        TYPE_LOCK_NOW: "NOW",
        TYPE_LOCK_ON_CREATE: "ON_CREATE",
        TYPE_LOCK_ON_VAR_STATE: "ON_VAR_STATE",
    }

    def __init__(self):
        self.Version = VariablePolicyEntry.ENTRY_REVISION
        self.Size = VariablePolicyEntry._HdrStructSize
        self.OffsetToName = self.Size
        self.Namespace = uuid.UUID(bytes=b'\x00' * 16)
        self.MinSize = VariablePolicyEntry.NO_MIN_SIZE
        self.MaxSize = VariablePolicyEntry.NO_MAX_SIZE
        self.AttributesMustHave = VariablePolicyEntry.NO_MUST_ATTR
        self.AttributesCantHave = VariablePolicyEntry.NO_CANT_ATTR
        self.LockPolicyType = VariablePolicyEntry.TYPE_NO_LOCK
        self.LockPolicy = None
        self.Name = None

    def __str__(self):
        result = "VARIABLE_POLICY_ENTRY(%s, %s)\n" % (self.Namespace, self.Name)

        if self.LockPolicyType in (VariablePolicyEntry.TYPE_NO_LOCK,
                                   VariablePolicyEntry.TYPE_LOCK_NOW,
                                   VariablePolicyEntry.TYPE_LOCK_ON_CREATE):
            result += "\tLock        = %s\n" % VariablePolicyEntry.LOCK_POLICY_STRING_MAP[self.LockPolicyType]
        elif self.LockPolicyType is VariablePolicyEntry.TYPE_LOCK_ON_VAR_STATE:
            result += "\tLock        = %s\n" % self.LockPolicy

        result += "\tMin = 0x%08X, Max = 0x%08X, Must = 0x%08X, Cant = 0x%08X\n" % (
            self.MinSize, self.MaxSize, self.AttributesMustHave, self.AttributesCantHave)

        return result

    @staticmethod
    def csv_header():
        """returns a list containing the names of the ordered columns that are produced by csv_row()"""
        return ['Namespace', 'Name', 'LockPolicyType', 'VarStateNamespace', 'VarStateName',
                'VarStateValue', 'MinSize', 'MaxSize', 'AttributesMustHave', 'AttributesCantHave']

    def csv_row(self, guid_xref: dict = None):
        """
        returns a list containing the elements of this structure (in the same order as the csv_header)
        ready to be written to a csv file

        guid_xref - a dictionary of GUID/name substitutions where the key is a uuid object
                    and the value is a string
        """
        if guid_xref is None:
            guid_xref = {}

        result = [guid_xref.get(self.Namespace, self.Namespace),
                  self.Name, VariablePolicyEntry.LOCK_POLICY_STRING_MAP[self.LockPolicyType]]

        if self.LockPolicyType in (VariablePolicyEntry.TYPE_NO_LOCK,
                                   VariablePolicyEntry.TYPE_LOCK_NOW,
                                   VariablePolicyEntry.TYPE_LOCK_ON_CREATE):
            result += ['N/A', 'N/A', 'N/A']
        elif self.LockPolicyType is VariablePolicyEntry.TYPE_LOCK_ON_VAR_STATE:
            result += [guid_xref.get(self.LockPolicy.Namespace, self.LockPolicy.Namespace),
                       self.LockPolicy.Name, self.LockPolicy.Value]

        result += ["0x%08X" % self.MinSize,
                   "0x%08X" % self.MaxSize,
                   str(EfiVariableAttributes(self.AttributesMustHave)),
                   str(EfiVariableAttributes(self.AttributesCantHave))]

        return result

    def decode(self, buffer):
        """
        load this object from a bytes buffer

        return any remaining buffer
        """
        (self.Version, self.Size, self.OffsetToName, _namespace,
            self.MinSize, self.MaxSize, self.AttributesMustHave,
            self.AttributesCantHave, self.LockPolicyType, _) = struct.unpack(
                self._HdrStructFormat, buffer[:self._HdrStructSize])

        if self.Version != VariablePolicyEntry.ENTRY_REVISION:
            raise ValueError("Unknown structure version!")
        if self.LockPolicyType not in VariablePolicyEntry.LOCK_POLICY_STRING_MAP:
            raise ValueError("Unknown LockPolicyType!")

        self.Namespace = uuid.UUID(bytes_le=_namespace)

        if self.OffsetToName != self.Size:
            self.Name = buffer[self.OffsetToName:self.Size].decode('utf-16').strip('\x00')

        if self.LockPolicyType == VariablePolicyEntry.TYPE_LOCK_ON_VAR_STATE:
            self.LockPolicy = VariableLockOnVarStatePolicy()
            self.LockPolicy.decode(buffer[self._HdrStructSize:self.OffsetToName])

        return buffer[self.Size:]



# @file path_utilities.py
# Code to help convert Edk2, absolute, and relative file paths
#
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##
import os
import logging
import fnmatch
from typing import Iterable

#
# Class to help convert from absolute path to EDK2 build path
# using workspace and packagepath variables
#


class Edk2Path(object):

    def __init__(self, ws: os.PathLike, packagepathlist: Iterable[os.PathLike], error_on_invalid_pp: bool = True):
        """ An Edk2Path object is an object that can be used to resolve edk2 relative paths

        Args:
            ws: absolute path or cwd relative path of the workspace.
            packagespathlist: list of packages path.
                Entries can be Absolute path, workspace relative path, or CWD relative.
            error_on_invalid_pp: default value is True. If packages path value is invalid raise exception
        """

        self.WorkspacePath = ws
        self.logger = logging.getLogger("Edk2Path")
        if(not os.path.isabs(ws)):
            self.WorkspacePath = os.path.abspath(os.path.join(os.getcwd(), ws))

        if(not os.path.isdir(self.WorkspacePath)):
            self.logger.error("Workspace path invalid.  {0}".format(ws))
            raise Exception("Workspace path invalid.  {0}".format(ws))

        # Set PackagePath
        self.PackagePathList = list()
        for a in packagepathlist:
            if(os.path.isabs(a)):
                self.PackagePathList.append(a)
            else:
                # see if workspace relative
                wsr = os.path.join(ws, a)
                if(os.path.isdir(wsr)):
                    self.PackagePathList.append(wsr)
                else:
                    # assume current working dir relative.  Will catch invalid dir when checking whole list
                    self.PackagePathList.append(os.path.abspath(os.path.join(os.getcwd(), a)))

        error = False
        for a in self.PackagePathList[:]:
            if(not os.path.isdir(a)):
                self.logger.log(logging.ERROR if error_on_invalid_pp else logging.WARNING,
                                "Invalid package path entry {0}".format(a))
                self.PackagePathList.remove(a)  # remove invalid path
                error = True

        # report error
        if(error and error_on_invalid_pp):
            raise Exception("Invalid package path directory(s)")

    def GetEdk2RelativePathFromAbsolutePath(self, abspath):
        ''' Given an absolute path return a edk2 path relative
        to workspace or packagespath.

        If not valid return None
        '''

        relpath = None
        found = False
        if abspath is None:
            return None
        for a in (os.path.normcase(p) for p in self.PackagePathList):
            if os.path.normcase(abspath).startswith(a):
                # found our path...now use original strings to avoid
                # change in case
                relpath = abspath[len(a):]
                found = True
                self.logger.debug("Successfully converted AbsPath to Edk2Relative Path using PackagePath")
                self.logger.debug("AbsolutePath: %s found in PackagePath: %s" % (abspath, a))
                break

        if(not found):
            # Does path start with workspace
            if os.path.normcase(abspath).startswith(os.path.normcase(self.WorkspacePath)):
                # found our path...now use original strings to avoid
                # change in case
                relpath = abspath[len(self.WorkspacePath):]
                found = True
                self.logger.debug("Successfully converted AbsPath to Edk2Relative Path using WorkspacePath")
                self.logger.debug("AbsolutePath: %s found in Workspace: %s" % (abspath, self.WorkspacePath))

        if(found):
            relpath = relpath.replace(os.sep, "/")
            return relpath.lstrip("/")

        # didn't find the path for conversion.
        self.logger.error("Failed to convert AbsPath to Edk2Relative Path")
        self.logger.error("AbsolutePath: %s" % abspath)
        return None

    def GetAbsolutePathOnThisSystemFromEdk2RelativePath(self, relpath, log_errors=True):
        ''' Given a edk2 relative path return an absolute path to the file
        in this workspace.

        Note: For case insensitive operating systems the case of the input
        relpath will be used for the return value even if it doesn't match
        the case in the filesystem.

        If not valid or doesn't exist return None'''

        if relpath is None:
            return None
        relpath = relpath.replace("/", os.sep)
        abspath = os.path.join(self.WorkspacePath, relpath)
        if os.path.exists(abspath):
            return abspath

        for a in self.PackagePathList:
            abspath = os.path.join(a, relpath)
            if(os.path.exists(abspath)):
                return abspath
        if log_errors:
            self.logger.error("Failed to convert Edk2Relative Path to an Absolute Path on this system.")
            self.logger.error("Relative Path: %s" % relpath)

        return None

    # Find the package this path belongs to using
    # some Heuristic.  This isn't perfect but at least
    # identifies the directory consistently
    #
    # @param InputPath:  absolute path to module
    #
    # @ret Name of Package that the module is in.
    def GetContainingPackage(self, InputPath):
        self.logger.debug("GetContainingPackage: %s" % InputPath)

        # Make a list that has the path case normalized for comparison.
        # This only does anything on Windows
        NormCasePackagesPathList = [os.path.normcase(x) for x in self.PackagePathList]

        # check InputPath to make sure it is at least in the folder structure of the code tree
        if os.path.normcase(self.WorkspacePath) not in os.path.normcase(InputPath):
            # not in workspace - check all the packages paths
            found_in_pp = False
            for p in NormCasePackagesPathList:
                if p in os.path.normcase(InputPath):
                    found_in_pp = True
                    break
            if(not found_in_pp):
                self.logger.error(f"{InputPath} not in code tree")
                self.logger.info("PackagePath is: %s" % os.pathsep.join(self.PackagePathList))
                self.logger.info("Workspace path is : %s" % self.WorkspacePath)
                return None

        # InputPath is in workspace or PackagesPath for worst case scenario.

        dirpathprevious = os.path.dirname(InputPath)
        dirpath = os.path.dirname(InputPath)
        for _ in range(100):  # 100 is just a counter to avoid infinite loops.  Path nodes are unlikely to exceed 100
            #
            # Check for a DEC file in this folder
            # if here then return the directory name as the "package"
            #
            for f in os.listdir(dirpath):
                if fnmatch.fnmatch(f.lower(), '*.dec'):
                    a = os.path.basename(dirpath)
                    self.logger.debug("Found DEC file at %s.  Pkg is: %s", dirpath, a)
                    return a

            #
            # if at the root of the workspace return the previous dir.
            # this catches cases where a package has no DEC
            #
            if os.path.normcase(dirpath) == os.path.normcase(self.WorkspacePath):
                a = os.path.basename(dirpathprevious)
                self.logger.debug("Reached Workspace Path.  Using previous directory: %s" % a)
                return a

            #
            # if at the root of a packagepath return the previous dir.
            # this catches cases where a package has no DEC
            #
            if os.path.normcase(dirpath) in NormCasePackagesPathList:
                a = os.path.basename(dirpathprevious)
                self.logger.debug("Reached Package Path.  Using previous directory: %s" % a)
                return a

            dirpathprevious = dirpath
            dirpath = os.path.dirname(dirpath)

        self.logger.error("Failed to find containing package for %s" % InputPath)
        self.logger.info("PackagePath is: %s" % os.pathsep.join(self.PackagePathList))
        self.logger.info("Workspace path is : %s" % self.WorkspacePath)
        return None

    # Find the list of modules (infs) that file path is in
    #
    # for now just assume any inf in the same dir or if none
    # then check parent dir.  If InputPath is not in the filesystem
    # this function will try to return the likely containing module
    # but if the entire module has been deleted this isn't possible.
    #
    # @param InputPath:  absolute path to file
    #
    # @ret list of abs path file paths for module infs
    def GetContainingModules(self, InputPath: str) -> list:
        self.logger.debug("GetContainingModules: %s" % InputPath)

        # if INF return self
        if fnmatch.fnmatch(InputPath.lower(), '*.inf'):
            return [InputPath]

        # Before checking the local filesystem for an INF
        # make sure filesystem has file or at least folder
        if not os.path.isfile(InputPath):
            logging.debug("InputPath doesn't exist in filesystem")

        modules = []
        # Check current dir
        dirpath = os.path.dirname(InputPath)
        if os.path.isdir(dirpath):
            for f in os.listdir(dirpath):
                if fnmatch.fnmatch(f.lower(), '*.inf'):
                    self.logger.debug("Found INF file in %s.  INf is: %s", dirpath, f)
                    modules.append(os.path.join(dirpath, f))

        # if didn't find any in current dir go to parent dir.
        # this handles cases like:
        # ModuleDir/
        #   Module.inf
        #   x64/
        #     file.c
        #
        if(len(modules) == 0):
            dirpath = os.path.dirname(dirpath)
            if os.path.isdir(dirpath):
                for f in os.listdir(dirpath):
                    if fnmatch.fnmatch(f.lower(), '*.inf'):
                        self.logger.debug("Found INF file in %s.  INf is: %s", dirpath, f)
                        modules.append(os.path.join(dirpath, f))

        return modules



## @file
# Module that encodes and decodes a EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER with
# a payload.
#
# Copyright (c) 2018 - 2019, Intel Corporation. All rights reserved.<BR>
# SPDX-License-Identifier: BSD-2-Clause-Patent
#

'''
FmpCapsuleHeader
'''

import struct
import uuid
from edk2toollib.uefi.fmp_auth_header import FmpAuthHeaderClass


class FmpCapsuleImageHeaderClass (object):
    # typedef struct {
    #   UINT32   Version;
    #
    #   ///
    #   /// Used to identify device firmware targeted by this update. This guid is matched by
    #   /// system firmware against ImageTypeId field within a EFI_FIRMWARE_IMAGE_DESCRIPTOR
    #   ///
    #   EFI_GUID UpdateImageTypeId;
    #
    #   ///
    #   /// Passed as ImageIndex in call to EFI_FIRMWARE_MANAGEMENT_PROTOCOL.SetImage ()
    #   ///
    #   UINT8    UpdateImageIndex;
    #   UINT8    reserved_bytes[3];
    #
    #   ///
    #   /// Size of the binary update image which immediately follows this structure
    #   ///
    #   UINT32   UpdateImageSize;
    #
    #   ///
    #   /// Size of the VendorCode bytes which optionally immediately follow binary update image in the capsule
    #   ///
    #   UINT32   UpdateVendorCodeSize;
    #
    #   ///
    #   /// The HardwareInstance to target with this update. If value is zero it means match all
    #   /// HardwareInstances. This field allows update software to target only a single device in
    #   /// cases where there are more than one device with the same ImageTypeId GUID.
    #   /// This header is outside the signed data of the Authentication Info structure and
    #   /// therefore can be modified without changing the Auth data.
    #   ///
    #   UINT64   UpdateHardwareInstance;
    # } EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER;
    #
    #  #define EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER_INIT_VERSION 0x00000002

    _StructFormat = '<I16sB3BIIQ'  # spell-checker: disable-line
    _StructSize = struct.calcsize(_StructFormat)

    EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER_INIT_VERSION = 0x00000002

    def __init__(self):
        self.Version = self.EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER_INIT_VERSION
        self.UpdateImageTypeId = uuid.UUID('00000000-0000-0000-0000-000000000000')
        self.UpdateImageIndex = 0
        self.UpdateImageSize = 0
        self.UpdateVendorCodeSize = 0
        self.UpdateHardwareInstance = 0x0000000000000000
        self.Payload = b''
        self.VendorCodeBytes = b''
        self.FmpAuthHeader = None

    def Encode(self):
        # If we have an FmpAuthHeader, let's collapse that now.
        if self.FmpAuthHeader is not None:
            self.Payload = self.FmpAuthHeader.Encode()

        self.UpdateImageSize = len(self.Payload)
        self.UpdateVendorCodeSize = len(self.VendorCodeBytes)
        FmpCapsuleImageHeader = struct.pack(
            self._StructFormat,
            self.Version,
            self.UpdateImageTypeId.bytes_le,
            self.UpdateImageIndex,
            0, 0, 0,
            self.UpdateImageSize,
            self.UpdateVendorCodeSize,
            self.UpdateHardwareInstance
        )
        return FmpCapsuleImageHeader + self.Payload + self.VendorCodeBytes

    def Decode(self, Buffer):
        if len(Buffer) < self._StructSize:
            raise ValueError
        (Version, UpdateImageTypeId, UpdateImageIndex, r0, r1, r2,
            UpdateImageSize, UpdateVendorCodeSize, UpdateHardwareInstance) = struct.unpack(
                self._StructFormat,
                Buffer[0:self._StructSize]
        )

        if Version < self.EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER_INIT_VERSION:
            raise ValueError
        if UpdateImageIndex < 1:
            raise ValueError
        if UpdateImageSize + UpdateVendorCodeSize != len(Buffer[self._StructSize:]):
            raise ValueError

        self.Version = Version
        self.UpdateImageTypeId = uuid.UUID(bytes_le=UpdateImageTypeId)
        self.UpdateImageIndex = UpdateImageIndex
        self.UpdateImageSize = UpdateImageSize
        self.UpdateVendorCodeSize = UpdateVendorCodeSize
        self.UpdateHardwareInstance = UpdateHardwareInstance
        self.Payload = Buffer[self._StructSize:self._StructSize + UpdateImageSize]
        if len(self.Payload) > 0:
            self.FmpAuthHeader = FmpAuthHeaderClass()
            self.FmpAuthHeader.Decode(self.Payload)
        self.VendorCodeBytes = Buffer[self._StructSize + UpdateImageSize:]
        return Buffer[self._StructSize:]

    def DumpInfo(self):
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER.Version                = {Version:08X}'
              .format(Version=self.Version))
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER.UpdateImageTypeId      = {UpdateImageTypeId}'
              .format(UpdateImageTypeId=str(self.UpdateImageTypeId).upper()))
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER.UpdateImageIndex       = {UpdateImageIndex:08X}'
              .format(UpdateImageIndex=self.UpdateImageIndex))
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER.UpdateImageSize        = {UpdateImageSize:08X}'
              .format(UpdateImageSize=self.UpdateImageSize))
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER.UpdateVendorCodeSize   = {UpdateVendorCodeSize:08X}'
              .format(UpdateVendorCodeSize=self.UpdateVendorCodeSize))
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_IMAGE_HEADER.UpdateHardwareInstance = {UpdateHardwareInstance:016X}'
              .format(UpdateHardwareInstance=self.UpdateHardwareInstance))
        print('sizeof (Payload)                                                    = {Size:08X}'
              .format(Size=len(self.Payload)))
        print('sizeof (VendorCodeBytes)                                            = {Size:08X}'
              .format(Size=len(self.VendorCodeBytes)))
        if self.FmpAuthHeader is not None:
            self.FmpAuthHeader.DumpInfo()


class FmpCapsuleHeaderClass (object):
    # typedef struct {
    #   UINT32 Version;
    #
    #   ///
    #   /// The number of drivers included in the capsule and the number of corresponding
    #   /// offsets stored in ItemOffsetList array.
    #   ///
    #   UINT16 EmbeddedDriverCount;
    #
    #   ///
    #   /// The number of payload items included in the capsule and the number of
    #   /// corresponding offsets stored in the ItemOffsetList array.
    #   ///
    #   UINT16 PayloadItemCount;
    #
    #   ///
    #   /// Variable length array of dimension [EmbeddedDriverCount + PayloadItemCount]
    #   /// containing offsets of each of the drivers and payload items contained within the capsule
    #   ///
    #   // UINT64 ItemOffsetList[];
    # } EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER;
    #
    #  #define EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER_INIT_VERSION       0x00000001
    _StructFormat = '<IHH'
    _StructSize = struct.calcsize(_StructFormat)

    _ItemOffsetFormat = '<Q'
    _ItemOffsetSize = struct.calcsize(_ItemOffsetFormat)

    EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER_INIT_VERSION = 0x00000001

    def __init__(self):
        self.Version = self.EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER_INIT_VERSION
        self.EmbeddedDriverCount = 0
        self.PayloadItemCount = 0
        self._EmbeddedDriverList = []
        self._FmpCapsuleImageHeaderList = []

    def AddEmbeddedDriver(self, EmbeddedDriver):
        self._EmbeddedDriverList.append(EmbeddedDriver)
        self.EmbeddedDriverCount += 1

    def GetEmbeddedDriver(self, Index):
        return self._EmbeddedDriverList[Index]

    def AddFmpCapsuleImageHeader(self, FmpCapsuleHeader):
        self._FmpCapsuleImageHeaderList.append(FmpCapsuleHeader)
        self.PayloadItemCount += 1

    def GetFmpCapsuleImageHeader(self, Index):
        return self._FmpCapsuleImageHeaderList[Index]

    def Encode(self):
        self.EmbeddedDriverCount = len(self._EmbeddedDriverList)
        self.PayloadItemCount = len(self._FmpCapsuleImageHeaderList)

        FmpCapsuleHeader = struct.pack(
            self._StructFormat,
            self.Version,
            self.EmbeddedDriverCount,
            self.PayloadItemCount
        )

        FmpCapsuleData = b''
        offset_list = []
        Offset = self._StructSize + (self.EmbeddedDriverCount + self.PayloadItemCount) * self._ItemOffsetSize
        for EmbeddedDriver in self._EmbeddedDriverList:
            FmpCapsuleData = FmpCapsuleData + EmbeddedDriver
            offset_list.append(Offset)
            Offset = Offset + len(EmbeddedDriver)
        for FmpCapsuleImageHeader in self._FmpCapsuleImageHeaderList:
            FmpCapsuleImage = FmpCapsuleImageHeader.Encode()
            FmpCapsuleData = FmpCapsuleData + FmpCapsuleImage
            offset_list.append(Offset)
            Offset = Offset + len(FmpCapsuleImage)

        for Offset in offset_list:
            FmpCapsuleHeader = FmpCapsuleHeader + struct.pack(self._ItemOffsetFormat, Offset)

        return FmpCapsuleHeader + FmpCapsuleData

    def Decode(self, Buffer):
        if len(Buffer) < self._StructSize:
            raise ValueError
        (Version, EmbeddedDriverCount, PayloadItemCount) = struct.unpack(
            self._StructFormat,
            Buffer[0:self._StructSize]
        )
        if Version < self.EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER_INIT_VERSION:
            raise ValueError

        self.Version = Version
        self.EmbeddedDriverCount = EmbeddedDriverCount
        self.PayloadItemCount = PayloadItemCount
        self._EmbeddedDriverList = []
        self._FmpCapsuleImageHeaderList = []

        offset_list = []

        #
        # Parse the ItemOffsetList values
        #
        Offset = self._StructSize
        for Index in range(0, EmbeddedDriverCount + PayloadItemCount):
            ItemOffset = struct.unpack(self._ItemOffsetFormat, Buffer[Offset:Offset + self._ItemOffsetSize])[0]
            if ItemOffset >= len(Buffer):
                raise ValueError
            offset_list.append(ItemOffset)
            Offset = Offset + self._ItemOffsetSize
        Result = Buffer[Offset:]

        #
        # Parse the EmbeddedDrivers
        #
        for Index in range(0, EmbeddedDriverCount):
            Offset = offset_list[Index]
            if Index < (len(offset_list) - 1):
                Length = offset_list[Index + 1] - Offset
            else:
                Length = len(Buffer) - Offset
            self.AddEmbeddedDriver(Buffer[Offset:Offset + Length])

        #
        # Parse the Payloads that are FMP Capsule Images
        #
        for Index in range(EmbeddedDriverCount, EmbeddedDriverCount + PayloadItemCount):
            Offset = offset_list[Index]
            if Index < (len(offset_list) - 1):
                Length = offset_list[Index + 1] - Offset
            else:
                Length = len(Buffer) - Offset
            FmpCapsuleImageHeader = FmpCapsuleImageHeaderClass()
            FmpCapsuleImageHeader.Decode(Buffer[Offset:Offset + Length])
            self.AddFmpCapsuleImageHeader(FmpCapsuleImageHeader)

        return Result

    def DumpInfo(self):
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER.Version             = {Version:08X}'.format(Version=self.Version))
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER.EmbeddedDriverCount = {EmbeddedDriverCount:08X}'.format(
            EmbeddedDriverCount=self.EmbeddedDriverCount))
        for EmbeddedDriver in self._EmbeddedDriverList:
            print('  sizeof (EmbeddedDriver)                                  = {Size:08X}'.format(
                Size=len(EmbeddedDriver)))
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER.PayloadItemCount    = {PayloadItemCount:08X}'.format(
            PayloadItemCount=self.PayloadItemCount))
        print('EFI_FIRMWARE_MANAGEMENT_CAPSULE_HEADER.ItemOffsetList      = ')
        for FmpCapsuleImageHeader in self._FmpCapsuleImageHeaderList:
            FmpCapsuleImageHeader.DumpInfo()



# @file dec_parser_test.py
# Contains unit test routines for the dec parser class.
#
#
# Copyright (c) Microsoft Corporation
#
# SPDX-License-Identifier: BSD-2-Clause-Patent
##

import unittest
import os
from edk2toollib.uefi.edk2.parsers.fdf_parser import FdfParser

TEST_PATH = os.path.realpath(os.path.dirname(__file__))


class TestBasicFdfParser(unittest.TestCase):

    def test_primary_defines(self):
        test_fdf = os.path.join(TEST_PATH, 'SimpleDefines.fdf')
        print(test_fdf)
        parser = FdfParser().SetBaseAbsPath(TEST_PATH)
        parser.ParseFile(test_fdf)

        # Make sure that we can read local variables out of the file.
        self.assertEqual(parser.Dict['FD_BASE'], '0x00800000')
        self.assertEqual(parser.Dict['NUM_BLOCKS'], '0x410')
        self.assertFalse("EXTRA_DEF" in parser.Dict)

    def test_primary_conditional_defines(self):
        test_fdf = os.path.join(TEST_PATH, 'SimpleDefines.fdf')
        print(test_fdf)
        parser = FdfParser().SetBaseAbsPath(TEST_PATH).SetInputVars({"TARGET": "TEST2"})
        parser.ParseFile(test_fdf)

        # Make sure that we can read local variables out of the file.
        self.assertEqual(parser.Dict['FD_BASE'], '0x00800000')
        self.assertEqual(parser.Dict['NUM_BLOCKS'], '0x850')
        self.assertTrue("EXTRA_DEF" in parser.Dict)

    def test_included_defines(self):
        test_fdf = os.path.join(TEST_PATH, 'IncludedDefinesParent.fdf')
        parser = FdfParser().SetBaseAbsPath(TEST_PATH)
        parser.ParseFile(test_fdf)

        # Make sure that we can read local variables out of the file.
        self.assertEqual(parser.Dict['FD_BASE'], '0x00800000')
        self.assertEqual(parser.Dict['EXTRA_BLOCK_SIZE'], '0x00001000')
        self.assertFalse("AM_I_YOU" in parser.Dict)

    def test_included_conditional_defines(self):
        test_fdf = os.path.join(TEST_PATH, 'IncludedDefinesParent.fdf')
        parser = FdfParser().SetBaseAbsPath(TEST_PATH)
        parser = FdfParser().SetBaseAbsPath(TEST_PATH).SetInputVars({"TARGET": "TEST4"})
        parser.ParseFile(test_fdf)

        # Make sure that we can read local variables out of the file.
        self.assertEqual(parser.Dict['FD_BASE'], '0x00800000')
        self.assertEqual(parser.Dict['EXTRA_BLOCK_SIZE'], '0x00001000')
        self.assertEqual(parser.Dict['NUM_BLOCKS'], '0x410')
        self.assertTrue("AM_I_YOU" in parser.Dict)
        self.assertFalse("CONDITIONAL_VALUE" in parser.Dict)

    def test_conditionally_included_defines(self):
        test_fdf = os.path.join(TEST_PATH, 'IncludedDefinesParent.fdf')
        parser = FdfParser().SetBaseAbsPath(TEST_PATH)
        parser = FdfParser().SetBaseAbsPath(TEST_PATH).SetInputVars({"TARGET": "TEST5"})
        parser.ParseFile(test_fdf)

        # Make sure that we can read local variables out of the file.
        self.assertEqual(parser.Dict['FD_BASE'], '0x00800000')
        self.assertEqual(parser.Dict['INTERNAL_VALUE'], '104')
        self.assertEqual(parser.Dict['NUM_BLOCKS'], '0x410')
        self.assertEqual(parser.Dict['CONDITIONAL_VALUE'], '121')
